@@ -180,6 +180,8 @@ class PrimitiveModel(DataModel):
 
 
 @register_default(types.Opaque)
+@register_default(types.PyObject)
+@register_default(types.RawPointer)
 @register_default(types.NoneType)
 @register_default(types.Function)
 @register_default(types.Type)
@@ -191,10 +193,12 @@ class PrimitiveModel(DataModel):
 @register_default(types.Dummy)
 @register_default(types.ExceptionInstance)
 @register_default(types.ExternalFunction)
+@register_default(types.NumbaFunction)
 @register_default(types.Method)
 @register_default(types.Macro)
 @register_default(types.NumberClass)
 @register_default(types.DType)
+@register_default(types.ArrayFlags)
 class OpaqueModel(PrimitiveModel):
     """
     Passed as opaque pointers
@@ -261,7 +265,7 @@ class EphemeralArrayModel(PointerModel):
         return ir.ArrayType(self._pointee_be_type, self._fe_type.count)
 
     def as_data(self, builder, value):
-        values = [builder.load(cgutils.gep(builder, value, i))
+        values = [builder.load(cgutils.gep_inbounds(builder, value, i))
                   for i in range(self._fe_type.count)]
         return cgutils.pack_array(builder, values)
 
@@ -429,7 +433,7 @@ class StructModel(CompositeModel):
     def load_from_data_pointer(self, builder, ptr):
         values = []
         for i, model in enumerate(self._models):
-            elem_ptr = cgutils.gep(builder, ptr, 0, i)
+            elem_ptr = cgutils.gep_inbounds(builder, ptr, 0, i)
             val = model.load_from_data_pointer(builder, elem_ptr)
             values.append(val)
 
@@ -711,7 +715,6 @@ class CContiguousFlatIter(StructModel):
         ndim = array_type.ndim
         members = [('array', types.EphemeralPointer(array_type)),
                    ('stride', types.intp),
-                   ('pointer', types.EphemeralPointer(types.CPointer(dtype))),
                    ('index', types.EphemeralPointer(types.intp)),
                    ]
         if need_indices:
@@ -838,6 +841,14 @@ class GeneratorModel(CompositeModel):
     def from_return(self, builder, value):
         return value
 
+    def as_data(self, builder, value):
+        return builder.load(value)
+
+    def from_data(self, builder, value):
+        stack = cgutils.alloca_once(builder, value.type)
+        builder.store(value, stack)
+        return stack
+
 
 @register_default(types.ArrayCTypes)
 class ArrayCTypesModel(StructModel):
