@@ -4,24 +4,7 @@ import os
 import pprint
 from collections import defaultdict
 
-
-class RedefinedError(NameError):
-    pass
-
-
-class NotDefinedError(NameError):
-    def __init__(self, name, loc=None):
-        self.name = name
-        self.loc = loc
-
-    def __str__(self):
-        loc = "?" if self.loc is None else self.loc
-        return "{name!r} is not defined in {loc}".format(name=self.name,
-                                                         loc=self.loc)
-
-
-class VerificationError(Exception):
-    pass
+from .errors import NotDefinedError, RedefinedError, VerificationError
 
 
 class Loc(object):
