@@ -9,17 +9,7 @@ from llvmlite.llvmpy.core import Constant, Type, Builder
 
 from . import (_dynfunc, cgutils, config, funcdesc, generators, ir, types,
                typing, utils)
-
-
-class LoweringError(Exception):
-    def __init__(self, msg, loc):
-        self.msg = msg
-        self.loc = loc
-        super(LoweringError, self).__init__("%s\n%s" % (msg, loc.strformat()))
-
-
-class ForbiddenConstruct(LoweringError):
-    pass
+from .errors import LoweringError
 
 
 _VarArgItem = namedtuple("_VarArgItem", ("vararg", "index"))
@@ -58,10 +48,39 @@ class BaseLower(object):
     def init(self):
         pass
 
+    def init_pyapi(self):
+        """
+        Init the Python API and Environment Manager for the function being
+        lowered.
+        """
+        if self.pyapi is not None:
+            return
+        self.pyapi = self.context.get_python_api(self.builder)
+
+        # Store environment argument for later use
+        self.envarg = self.call_conv.get_env_argument(self.function)
+        # Sanity check
+        with cgutils.if_unlikely(self.builder,
+                                 cgutils.is_null(self.builder, self.envarg)):
+            self.pyapi.err_set_string(
+                "PyExc_SystemError",
+                "Numba internal error: object mode function called "
+                "without an environment")
+            self.call_conv.return_exc(self.builder)
+
+        self.env_body = self.context.get_env_body(self.builder, self.envarg)
+        self.pyapi.emit_environment_sentry(self.envarg)
+        self.env_manager = self.pyapi.get_env_manager(self.env, self.env_body,
+                                                      self.envarg)
+
     def pre_lower(self):
         """
         Called before lowering all blocks.
         """
+        # A given Lower object can be used for several LL functions
+        # (for generators) and it's important to use a new API and
+        # EnvironmentManager.
+        self.pyapi = None
 
     def post_lower(self):
         """
@@ -164,10 +183,10 @@ class BaseLower(object):
         if self.genlower:
             self.context.create_cpython_wrapper(self.library,
                                                 self.genlower.gendesc,
-                                                self.call_helper,
+                                                self.env, self.call_helper,
                                                 release_gil=release_gil)
         self.context.create_cpython_wrapper(self.library, self.fndesc,
-                                            self.call_helper,
+                                            self.env, self.call_helper,
                                             release_gil=release_gil)
 
     def setup_function(self, fndesc):
@@ -449,11 +468,16 @@ class Lower(BaseLower):
             # Handle a named external function
             fndesc = funcdesc.ExternalFunctionDescriptor(
                 fnty.symbol, fnty.sig.return_type, fnty.sig.args)
-            func = self.context.declare_external_function(
-                    cgutils.get_module(self.builder), fndesc)
+            func = self.context.declare_external_function(self.builder.module,
+                                                          fndesc)
             res = self.context.call_external_function(
                 self.builder, func, fndesc.argtypes, argvals)
 
+        elif isinstance(fnty, types.NumbaFunction):
+            # Handle a compiled Numba function
+            res = self.context.call_internal(self.builder, fnty.fndesc,
+                                             fnty.sig, argvals)
+
         elif isinstance(fnty, types.Method):
             # Method of objects are handled differently
             fnobj = self.loadvar(expr.func.name)
@@ -465,9 +489,9 @@ class Lower(BaseLower):
             pointer = self.loadvar(expr.func.name)
             # If the external function pointer uses libpython
             if fnty.requires_gil:
-                pyapi = self.context.get_python_api(self.builder)
+                self.init_pyapi()
                 # Acquire the GIL
-                gil_state = pyapi.gil_ensure()
+                gil_state = self.pyapi.gil_ensure()
                 # Make PyObjects
                 newargvals = []
                 pyvals = []
@@ -475,7 +499,8 @@ class Lower(BaseLower):
                                                 argvals):
                     # Adjust argument values to pyobjects
                     if exptyp == types.ffi_forced_object:
-                        obj = pyapi.from_native_value(aval, gottyp)
+                        obj = self.pyapi.from_native_value(aval, gottyp,
+                                                           self.env_manager)
                         newargvals.append(obj)
                         pyvals.append(obj)
                     else:
@@ -486,10 +511,10 @@ class Lower(BaseLower):
                                                          newargvals, fnty.cconv)
                 # Release PyObjects
                 for obj in pyvals:
-                    pyapi.decref(obj)
+                    self.pyapi.decref(obj)
 
                 # Release the GIL
-                pyapi.gil_release(gil_state)
+                self.pyapi.gil_release(gil_state)
             # If the external function pointer does NOT use libpython
             else:
                 res = self.context.call_function_pointer(self.builder, pointer,
