@@ -1,6 +1,7 @@
 from __future__ import print_function, division, absolute_import
+import inspect
 
-from . import _internal
+from . import _internal, dufunc
 from .ufuncbuilder import UFuncBuilder, GUFuncBuilder
 
 from numba.targets.registry import TargetRegistry
@@ -39,17 +40,28 @@ class GUVectorize(_BaseVectorize):
         return imp(func, signature, identity, kws)
 
 
-def vectorize(ftylist, **kws):
-    """vectorize(ftylist, target='cpu', identity=None, **kws)
+def vectorize(ftylist_or_function=(), **kws):
+    """vectorize(ftylist_or_function=(), target='cpu', identity=None, **kws)
 
-    A decorator to create numpy ufunc object from Numba compiled code.
+    A decorator that creates a Numpy ufunc object using Numba compiled
+    code.  When no arguments or only keyword arguments are given,
+    vectorize will return a Numba dynamic ufunc (DUFunc) object, where
+    compilation/specialization may occur at call-time.
 
     Args
     -----
-    ftylist: iterable
-        An iterable of type signatures, which are either
-        function type object or a string describing the
-        function type.
+    ftylist_or_function: function or iterable
+
+        When the first argument is a function, signatures are dealt
+        with at call-time.
+
+        When the first argument is an iterable of type signatures,
+        which are either function type object or a string describing
+        the function type, signatures are finalized at decoration
+        time.
+
+    Keyword Args
+    ------------
 
     target: str
             A string for code generation target.  Default to "cpu".
@@ -64,23 +76,37 @@ def vectorize(ftylist, **kws):
 
     A NumPy universal function
 
-    Example
+    Examples
     -------
         @vectorize(['float32(float32, float32)',
                     'float64(float64, float64)'], identity=1)
         def sum(a, b):
             return a + b
 
+        @vectorize
+        def sum(a, b):
+            return a + b
+
+        @vectorize(identity=1)
+        def mul(a, b):
+            return a * b
+
     """
-    if isinstance(ftylist, str):
+    if isinstance(ftylist_or_function, str):
         # Common user mistake
-        ftylist = [ftylist]
+        ftylist = [ftylist_or_function]
+    elif inspect.isfunction(ftylist_or_function):
+        return dufunc.DUFunc(ftylist_or_function, **kws)
+    elif ftylist_or_function is not None:
+        ftylist = ftylist_or_function
 
     def wrap(func):
-        vec = Vectorize(func, **kws)
-        for fty in ftylist:
-            vec.add(fty)
-        return vec.build_ufunc()
+        if len(ftylist) > 0:
+            vec = Vectorize(func, **kws)
+            for fty in ftylist:
+                vec.add(fty)
+            return vec.build_ufunc()
+        return dufunc.DUFunc(func, **kws)
 
     return wrap
 
