@@ -18,6 +18,7 @@ class TLStack(object):
         except AttributeError:
             # Initialize stack for the thread
             self.local.stack = []
+            return self.local.stack
 
     def push(self, item):
         self.stack.append(item)
@@ -39,5 +40,8 @@ class TLStack(object):
     def __nonzero__(self):
         return self.__bool__()
 
+    def __len__(self):
+        return len(self.stack)
+
     def clear(self):
         self.__init__()
