@@ -23,12 +23,77 @@ from numba.targets.imputils import (builtin, builtin_attr, implement,
 from .builtins import Slice
 
 
+def increment_index(builder, val):
+    """
+    Increment an index *val*.
+    """
+    one = Constant.int(val.type, 1)
+    # We pass the "nsw" flag in the hope that LLVM understands the index
+    # never changes sign.  Unfortunately this doesn't always work
+    # (e.g. ndindex()).
+    return builder.add(val, one, flags=['nsw'])
+
+
+def set_range_metadata(builder, load, lower_bound, upper_bound):
+    """
+    Set the "range" metadata on a load instruction.
+    Note the interval is in the form [lower_bound, upper_bound).
+    """
+    range_operands = [Constant.int(load.type, lower_bound),
+                      Constant.int(load.type, upper_bound)]
+    md = builder.module.add_metadata(range_operands)
+    load.set_metadata("range", md)
+
+
+def mark_positive(builder, load):
+    """
+    Mark the result of a load instruction as positive (or zero).
+    """
+    upper_bound = (1 << (load.type.width - 1)) - 1
+    set_range_metadata(builder, load, 0, upper_bound)
+
+
 def make_array(array_type):
     """
     Return the Structure representation of the given *array_type*
     (an instance of types.Array).
     """
-    return cgutils.create_struct_proxy(array_type)
+    base = cgutils.create_struct_proxy(array_type)
+    ndim = array_type.ndim
+
+    class ArrayStruct(base):
+        @property
+        def shape(self):
+            """
+            Override .shape to inform LLVM that its elements are all positive.
+            """
+            builder = self._builder
+            if ndim == 0:
+                return base.__getattr__(self, "shape")
+
+            # Unfortunately, we can't use llvm.assume as its presence can
+            # seriously pessimize performance,
+            # *and* the range metadata currently isn't improving anything here,
+            # see https://llvm.org/bugs/show_bug.cgi?id=23848 !
+            ptr = self._get_ptr_by_name("shape")
+            dims = []
+            for i in range(ndim):
+                dimptr = cgutils.gep_inbounds(builder, ptr, 0, i)
+                load = builder.load(dimptr)
+                dims.append(load)
+                mark_positive(builder, load)
+
+            return cgutils.pack_array(builder, dims)
+
+    return ArrayStruct
+
+
+def get_itemsize(context, array_type):
+    """
+    Return the item size for the given array or buffer type.
+    """
+    llty = context.get_data_type(array_type.dtype)
+    return context.get_abi_sizeof(llty)
 
 
 def populate_array(array, data, shape, strides, itemsize, meminfo,
@@ -67,7 +132,7 @@ def populate_array(array, data, shape, strides, itemsize, meminfo,
             nitems = builder.mul(nitems, axlen)
     else:
         # Shape is empty
-        nitems = context.get_constant(types.intp, 0)
+        nitems = context.get_constant(types.intp, 1)
     attrs['nitems'] = nitems
 
     # Make sure that we have all the fields
@@ -82,6 +147,25 @@ def populate_array(array, data, shape, strides, itemsize, meminfo,
     return array
 
 
+def update_array_info(aryty, array):
+    """
+    Update some auxiliary information in *array* after some of its fields
+    were changed.  `itemsize` and `nitems` are updated.
+    """
+    context = array._context
+    builder = array._builder
+
+    # Calc num of items from shape
+    nitems = context.get_constant(types.intp, 1)
+    unpacked_shape = cgutils.unpack_tuple(builder, array.shape, aryty.ndim)
+    for axlen in unpacked_shape:
+        nitems = builder.mul(nitems, axlen)
+    array.nitems = nitems
+
+    array.itemsize = context.get_constant(types.intp,
+                                          get_itemsize(context, aryty))
+
+
 @struct_factory(types.ArrayIterator)
 def make_arrayiter_cls(iterator_type):
     """
@@ -133,7 +217,7 @@ def iternext_array(context, builder, sig, args, result):
     is_valid = builder.icmp(lc.ICMP_SLT, index, nitems)
     result.set_valid(is_valid)
 
-    with cgutils.ifthen(builder, is_valid):
+    with builder.if_then(is_valid):
         value = _getitem_array1d(context, builder, arrayty, ary, index,
                                  wraparound=False)
         result.yield_(value)
@@ -149,9 +233,11 @@ def getitem_arraynd_intp(context, builder, sig, args):
     adapted_ary = arystty(context, builder, ary)
     ndim = aryty.ndim
     if ndim == 1:
+        # Return a value
         result = _getitem_array1d(context, builder, aryty, adapted_ary, idx,
                                   wraparound=idxty.signed)
     elif ndim > 1:
+        # Return a subview over the array
         out_ary_ty = make_array(aryty.copy(ndim = ndim - 1))
         out_ary = out_ary_ty(context, builder)
         in_shapes = cgutils.unpack_tuple(builder, adapted_ary.shape, count=ndim)
@@ -391,26 +477,26 @@ def setitem_array1d_slice(context, builder, sig, args):
 
     b_step_eq_zero = builder.icmp(lc.ICMP_EQ, slicestruct.step, ZERO)
     # bail if step is 0
-    with cgutils.ifthen(builder, b_step_eq_zero):
+    with builder.if_then(b_step_eq_zero):
         context.call_conv.return_user_exc(builder, ValueError,
                                           ("slice step cannot be zero",))
 
     # adjust for negative indices for start
     start = cgutils.alloca_once_value(builder, slicestruct.start)
     b_start_lt_zero = builder.icmp(lc.ICMP_SLT, builder.load(start), ZERO)
-    with cgutils.ifthen(builder, b_start_lt_zero):
+    with builder.if_then(b_start_lt_zero):
         add = builder.add(builder.load(start), shapes[0])
         builder.store(add, start)
 
     b_start_lt_zero = builder.icmp(lc.ICMP_SLT, builder.load(start), ZERO)
-    with cgutils.ifthen(builder, b_start_lt_zero):
+    with builder.if_then(b_start_lt_zero):
         b_step_lt_zero = builder.icmp(lc.ICMP_SLT, slicestruct.step, ZERO)
         cond = builder.select(b_step_lt_zero, NEG_ONE, ZERO)
         builder.store(cond, start)
 
     b_start_geq_len = builder.icmp(lc.ICMP_SGE, builder.load(start), shapes[0])
     ONE = Constant.int(shapes[0].type, 1)
-    with cgutils.ifthen(builder, b_start_geq_len):
+    with builder.if_then(b_start_geq_len):
         b_step_lt_zero = builder.icmp(lc.ICMP_SLT, slicestruct.step, ZERO)
         cond = builder.select(b_step_lt_zero, builder.sub(shapes[0], ONE), shapes[0])
         builder.store(cond, start)
@@ -418,25 +504,25 @@ def setitem_array1d_slice(context, builder, sig, args):
     # adjust stop for negative value
     stop = cgutils.alloca_once_value(builder, slicestruct.stop)
     b_stop_lt_zero = builder.icmp(lc.ICMP_SLT, builder.load(stop), ZERO)
-    with cgutils.ifthen(builder, b_stop_lt_zero):
+    with builder.if_then(b_stop_lt_zero):
         add = builder.add(builder.load(stop), shapes[0])
         builder.store(add, stop)
 
     b_stop_lt_zero = builder.icmp(lc.ICMP_SLT, builder.load(stop), ZERO)
-    with cgutils.ifthen(builder, b_stop_lt_zero):
+    with builder.if_then(b_stop_lt_zero):
         b_step_lt_zero = builder.icmp(lc.ICMP_SLT, slicestruct.step, ZERO)
         cond = builder.select(b_step_lt_zero, NEG_ONE, ZERO)
         builder.store(cond, start)
 
     b_stop_geq_len = builder.icmp(lc.ICMP_SGE, builder.load(stop), shapes[0])
     ONE = Constant.int(shapes[0].type, 1)
-    with cgutils.ifthen(builder, b_stop_geq_len):
+    with builder.if_then(b_stop_geq_len):
         b_step_lt_zero = builder.icmp(lc.ICMP_SLT, slicestruct.step, ZERO)
         cond = builder.select(b_step_lt_zero, builder.sub(shapes[0], ONE), shapes[0])
         builder.store(cond, stop)
 
     b_step_gt_zero = builder.icmp(lc.ICMP_SGT, slicestruct.step, ZERO)
-    with cgutils.ifelse(builder, b_step_gt_zero) as (then0, otherwise0):
+    with builder.if_else(b_step_gt_zero) as (then0, otherwise0):
         with then0:
             with cgutils.for_range_slice(builder, builder.load(start), builder.load(stop), slicestruct.step, slicestruct.start.type) as loop_idx1:
                 ptr = cgutils.get_item_pointer(builder, aryty, ary,
@@ -462,11 +548,177 @@ def array_len(context, builder, sig, args):
     return builder.extract_value(shapeary, 0)
 
 
+#-------------------------------------------------------------------------------
+# Shape / layout altering
+
+@builtin
+@implement('array.transpose', types.Kind(types.Array))
+def array_transpose(context, builder, sig, args):
+    return array_T(context, builder, sig.args[0], args[0])
+
+def array_T(context, builder, typ, value):
+    if typ.ndim <= 1:
+        return value
+    else:
+        ary = make_array(typ)(context, builder, value)
+        ret = make_array(typ)(context, builder)
+        shapes = cgutils.unpack_tuple(builder, ary.shape, typ.ndim)
+        strides = cgutils.unpack_tuple(builder, ary.strides, typ.ndim)
+        populate_array(ret,
+                       data=ary.data,
+                       shape=cgutils.pack_array(builder, shapes[::-1]),
+                       strides=cgutils.pack_array(builder, strides[::-1]),
+                       itemsize=ary.itemsize,
+                       meminfo=ary.meminfo,
+                       parent=ary.parent)
+        return ret._getvalue()
+
+builtin_attr(impl_attribute(types.Kind(types.Array), 'T')(array_T))
+
+
+def _attempt_nocopy_reshape(context, builder, aryty, ary, newnd, newshape,
+                            newstrides):
+    """
+    Call into Numba_attempt_nocopy_reshape() for the given array type
+    and instance, and the specified new shape.  The array pointed to
+    by *newstrides* will be filled up if successful.
+    """
+    ll_intp = context.get_value_type(types.intp)
+    ll_intp_star = ll_intp.as_pointer()
+    ll_intc = context.get_value_type(types.intc)
+    fnty = lc.Type.function(ll_intc, [ll_intp, ll_intp_star, ll_intp_star,
+                                      ll_intp, ll_intp_star, ll_intp_star,
+                                      ll_intp, ll_intc])
+    fn = builder.module.get_or_insert_function(
+        fnty, name="numba_attempt_nocopy_reshape")
+
+    nd = lc.Constant.int(ll_intp, aryty.ndim)
+    shape = cgutils.gep_inbounds(builder, ary._get_ptr_by_name('shape'), 0, 0)
+    strides = cgutils.gep_inbounds(builder, ary._get_ptr_by_name('strides'), 0, 0)
+    newnd = lc.Constant.int(ll_intp, newnd)
+    newshape = cgutils.gep_inbounds(builder, newshape, 0, 0)
+    newstrides = cgutils.gep_inbounds(builder, newstrides, 0, 0)
+    is_f_order = lc.Constant.int(ll_intc, 0)
+    res = builder.call(fn, [nd, shape, strides,
+                            newnd, newshape, newstrides,
+                            ary.itemsize, is_f_order])
+    return res
+
+@builtin
+@implement('array.reshape', types.Kind(types.Array), types.Kind(types.BaseTuple))
+def array_reshape(context, builder, sig, args):
+    aryty = sig.args[0]
+    retty = sig.return_type
+    shapety = sig.args[1]
+    shape = args[1]
+
+    ll_intp = context.get_value_type(types.intp)
+    ll_shape = lc.Type.array(ll_intp, shapety.count)
+
+    ary = make_array(aryty)(context, builder, args[0])
+
+    # XXX unknown dimension (-1) is unhandled
+
+    # Check requested size
+    newsize = lc.Constant.int(ll_intp, 1)
+    for s in cgutils.unpack_tuple(builder, shape):
+        newsize = builder.mul(newsize, s)
+    size = lc.Constant.int(ll_intp, 1)
+    for s in cgutils.unpack_tuple(builder, ary.shape):
+        size = builder.mul(size, s)
+    fail = builder.icmp_unsigned('!=', size, newsize)
+    with builder.if_then(fail):
+        msg = "total size of new array must be unchanged"
+        context.call_conv.return_user_exc(builder, ValueError, (msg,))
+
+    newnd = shapety.count
+    newshape = cgutils.alloca_once(builder, ll_shape)
+    builder.store(shape, newshape)
+    newstrides = cgutils.alloca_once(builder, ll_shape)
+
+    ok = _attempt_nocopy_reshape(context, builder, aryty, ary, newnd,
+                                 newshape, newstrides)
+    fail = builder.icmp_unsigned('==', ok, lc.Constant.int(ok.type, 0))
+
+    with builder.if_then(fail):
+        msg = "incompatible shape for array"
+        context.call_conv.return_user_exc(builder, NotImplementedError, (msg,))
+
+    ret = make_array(retty)(context, builder)
+    populate_array(ret,
+                   data=ary.data,
+                   shape=builder.load(newshape),
+                   strides=builder.load(newstrides),
+                   itemsize=ary.itemsize,
+                   meminfo=ary.meminfo,
+                   parent=ary.parent)
+    return ret._getvalue()
+
+
+def _change_dtype(context, builder, oldty, newty, ary):
+    """
+    Attempt to fix up *ary* for switching from *oldty* to *newty*.
+    Non-zero is returned on success.
+    """
+    assert oldty.ndim == newty.ndim
+    assert oldty.layout == newty.layout
+    ll_intp = context.get_value_type(types.intp)
+    ll_intp_star = ll_intp.as_pointer()
+    ll_intc = context.get_value_type(types.intc)
+    ll_char = context.get_value_type(types.int8)
+    fnty = lc.Type.function(ll_intc, [ll_intp, ll_intp_star, ll_intp_star,
+                                      ll_intp, ll_intp, ll_char])
+    fn = builder.module.get_or_insert_function(fnty,
+                                               name="numba_change_dtype")
+
+    old_itemsize = context.get_constant(types.intp, get_itemsize(context, oldty))
+    new_itemsize = context.get_constant(types.intp, get_itemsize(context, newty))
+
+    nd = lc.Constant.int(ll_intp, newty.ndim)
+    shape = cgutils.gep_inbounds(builder, ary._get_ptr_by_name('shape'), 0, 0)
+    strides = cgutils.gep_inbounds(builder, ary._get_ptr_by_name('strides'), 0, 0)
+    layout = lc.Constant.int(ll_char, ord(newty.layout))
+    res = builder.call(fn, [nd, shape, strides,
+                            old_itemsize, new_itemsize, layout])
+    update_array_info(newty, ary)
+    return res
+
+
+@builtin
+@implement('array.view', types.Kind(types.Array), types.Kind(types.DTypeSpec))
+def array_view(context, builder, sig, args):
+    aryty = sig.args[0]
+    retty = sig.return_type
+
+    ary = make_array(aryty)(context, builder, args[0])
+    ret = make_array(retty)(context, builder)
+    # Copy all fields, casting the "data" pointer appropriately
+    fields = set(ret._datamodel._fields)
+    for k in sorted(fields):
+        val = getattr(ary, k)
+        if k == 'data':
+            ptrty = ret.data.type
+            ret.data = builder.bitcast(val, ptrty)
+        else:
+            setattr(ret, k, val)
+
+    ok = _change_dtype(context, builder, aryty, retty, ret)
+    fail = builder.icmp_unsigned('==', ok, lc.Constant.int(ok.type, 0))
+
+    with builder.if_then(fail):
+        msg = "new type not compatible with array"
+        context.call_conv.return_user_exc(builder, ValueError, (msg,))
+
+    return ret._getvalue()
+
+
+#-------------------------------------------------------------------------------
+# Computations
+
 @builtin
 @implement(numpy.sum, types.Kind(types.Array))
 @implement("array.sum", types.Kind(types.Array))
 def array_sum(context, builder, sig, args):
-    [arrty] = sig.args
 
     def array_sum_impl(arr):
         c = 0
@@ -482,7 +734,6 @@ def array_sum(context, builder, sig, args):
 @implement(numpy.prod, types.Kind(types.Array))
 @implement("array.prod", types.Kind(types.Array))
 def array_prod(context, builder, sig, args):
-    [arrty] = sig.args
 
     def array_prod_impl(arr):
         c = 1
@@ -499,7 +750,7 @@ def array_prod(context, builder, sig, args):
 @implement("array.cumsum", types.Kind(types.Array))
 def array_cumsum(context, builder, sig, args):
     scalar_dtype = sig.return_type.dtype
-    dtype = getattr(numpy, str(scalar_dtype))
+    dtype = as_dtype(scalar_dtype)
 
     def array_cumsum_impl(arr):
         size = 1
@@ -519,9 +770,9 @@ def array_cumsum(context, builder, sig, args):
 @builtin
 @implement(numpy.cumprod, types.Kind(types.Array))
 @implement("array.cumprod", types.Kind(types.Array))
-def array_cumsum(context, builder, sig, args):
+def array_cumprod(context, builder, sig, args):
     scalar_dtype = sig.return_type.dtype
-    dtype = getattr(numpy, str(scalar_dtype))
+    dtype = as_dtype(scalar_dtype)
 
     def array_cumprod_impl(arr):
         size = 1
@@ -542,7 +793,6 @@ def array_cumsum(context, builder, sig, args):
 @implement(numpy.mean, types.Kind(types.Array))
 @implement("array.mean", types.Kind(types.Array))
 def array_mean(context, builder, sig, args):
-    [arrty] = sig.args
 
     def array_mean_impl(arr):
         # Can't use the naive `arr.sum() / arr.size`, as it would return
@@ -660,7 +910,7 @@ def _np_round_intrinsic(tp):
 
 def _np_round_float(context, builder, tp, val):
     llty = context.get_value_type(tp)
-    module = cgutils.get_module(builder)
+    module = builder.module
     fnty = lc.Type.function(llty, [llty])
     fn = module.get_or_insert_function(fnty, name=_np_round_intrinsic(tp))
     return builder.call(fn, (val,))
@@ -741,7 +991,12 @@ def array_round(context, builder, sig, args):
 
 
 #-------------------------------------------------------------------------------
+# Array attributes
 
+@builtin_attr
+@impl_attribute(types.Kind(types.Array), "dtype", types.Kind(types.DType))
+def array_dtype(context, builder, typ, value):
+    return context.get_dummy_value()
 
 @builtin_attr
 @impl_attribute(types.Kind(types.Array), "shape", types.Kind(types.UniTuple))
@@ -834,6 +1089,12 @@ def array_ctypes(context, builder, typ, value):
     return ctinfo._getvalue()
 
 
+@builtin_attr
+@impl_attribute(types.Kind(types.Array), "flags", types.Kind(types.ArrayFlags))
+def array_flags(context, builder, typ, value):
+    return context.get_dummy_value()
+
+
 @builtin_attr
 @impl_attribute(types.Kind(types.ArrayCTypes), "data", types.uintp)
 def array_ctypes_data(context, builder, typ, value):
@@ -842,14 +1103,34 @@ def array_ctypes_data(context, builder, typ, value):
     return ctinfo.data
 
 
+@builtin_attr
+@impl_attribute(types.Kind(types.ArrayFlags), "contiguous", types.boolean)
+@impl_attribute(types.Kind(types.ArrayFlags), "c_contiguous", types.boolean)
+def array_ctypes_data(context, builder, typ, value):
+    val = typ.array_type.layout == 'C'
+    return context.get_constant(types.boolean, val)
+
+@builtin_attr
+@impl_attribute(types.Kind(types.ArrayFlags), "f_contiguous", types.boolean)
+def array_ctypes_data(context, builder, typ, value):
+    layout = typ.array_type.layout
+    val = layout == 'F' if typ.array_type.ndim > 1 else layout in 'CF'
+    return context.get_constant(types.boolean, val)
+
+
 @builtin_attr
 @impl_attribute_generic(types.Kind(types.Array))
 def array_record_getattr(context, builder, typ, value, attr):
+    """
+    Generic getattr() implementation for record arrays: fetch the given
+    record member.
+    """
     arrayty = make_array(typ)
     array = arrayty(context, builder, value)
 
     rectype = typ.dtype
-    assert isinstance(rectype, types.Record)
+    if not isinstance(rectype, types.Record):
+        raise AttributeError("attribute %r of %s not defined" % (attr, typ))
     dtype = rectype.typeof(attr)
     offset = rectype.offset(attr)
 
@@ -901,14 +1182,14 @@ def _increment_indices(context, builder, ndim, shape, indices, end_flag=None):
     zero = context.get_constant(types.intp, 0)
     one = context.get_constant(types.intp, 1)
 
-    bbend = cgutils.append_basic_block(builder, 'end_increment')
+    bbend = builder.append_basic_block('end_increment')
 
     if end_flag is not None:
         builder.store(cgutils.false_byte, end_flag)
 
     for dim in reversed(range(ndim)):
-        idxptr = cgutils.gep(builder, indices, dim)
-        idx = builder.add(builder.load(idxptr), one)
+        idxptr = cgutils.gep_inbounds(builder, indices, dim)
+        idx = increment_index(builder, builder.load(idxptr))
 
         count = shape[dim]
         in_bounds = builder.icmp(lc.ICMP_SLT, idx, count)
@@ -949,7 +1230,7 @@ def make_ndindex_cls(nditerty):
             exhausted = cgutils.alloca_once_value(builder, cgutils.false_byte)
 
             for dim in range(ndim):
-                idxptr = cgutils.gep(builder, indices, dim)
+                idxptr = cgutils.gep_inbounds(builder, indices, dim)
                 builder.store(zero, idxptr)
                 # 0-sized dimensions really indicate an empty array,
                 # but we have to catch that condition early to avoid
@@ -961,22 +1242,25 @@ def make_ndindex_cls(nditerty):
 
             self.indices = indices
             self.exhausted = exhausted
-            self.shape = cgutils.pack_array(builder, shapes)
+            self.shape = cgutils.pack_array(builder, shapes, zero.type)
 
         def iternext_specific(self, context, builder, result):
             zero = context.get_constant(types.intp, 0)
             one = context.get_constant(types.intp, 1)
 
-            bbend = cgutils.append_basic_block(builder, 'end')
+            bbend = builder.append_basic_block('end')
 
             exhausted = cgutils.as_bool_bit(builder, builder.load(self.exhausted))
             with cgutils.if_unlikely(builder, exhausted):
                 result.set_valid(False)
                 builder.branch(bbend)
 
-            indices = [builder.load(cgutils.gep(builder, self.indices, dim))
+            indices = [builder.load(cgutils.gep_inbounds(builder, self.indices, dim))
                        for dim in range(ndim)]
-            result.yield_(cgutils.pack_array(builder, indices))
+            for load in indices:
+                mark_positive(builder, load)
+
+            result.yield_(cgutils.pack_array(builder, indices, zero.type))
             result.set_valid(True)
 
             shape = cgutils.unpack_tuple(builder, self.shape, ndim)
@@ -1004,7 +1288,6 @@ def _make_flattening_iter_cls(flatiterty, kind):
             def init_specific(self, context, builder, arrty, arr):
                 zero = context.get_constant(types.intp, 0)
                 self.index = cgutils.alloca_once_value(builder, zero)
-                self.pointer = cgutils.alloca_once_value(builder, arr.data)
                 # We can't trust strides[-1] to always contain the right
                 # step value, see
                 # http://docs.scipy.org/doc/numpy-dev/release.html#npy-relaxed-strides-checking
@@ -1017,11 +1300,16 @@ def _make_flattening_iter_cls(flatiterty, kind):
                         size=context.get_constant(types.intp, arrty.ndim))
 
                     for dim in range(arrty.ndim):
-                        idxptr = cgutils.gep(builder, indices, dim)
+                        idxptr = cgutils.gep_inbounds(builder, indices, dim)
                         builder.store(zero, idxptr)
 
                     self.indices = indices
 
+            # NOTE: Using gep() instead of explicit pointer addition helps
+            # LLVM vectorize the loop (since the stride is known and
+            # constant).  This is not possible in the non-contiguous case,
+            # where the strides are unknown at compile-time.
+
             def iternext_specific(self, context, builder, arrty, arr, result):
                 zero = context.get_constant(types.intp, 0)
                 one = context.get_constant(types.intp, 1)
@@ -1034,14 +1322,14 @@ def _make_flattening_iter_cls(flatiterty, kind):
                 result.set_valid(is_valid)
 
                 with cgutils.if_likely(builder, is_valid):
-                    ptr = builder.load(self.pointer)
+                    ptr = builder.gep(arr.data, [index])
                     value = context.unpack_value(builder, arrty.dtype, ptr)
                     if kind == 'flat':
                         result.yield_(value)
                     else:
                         # ndenumerate(): fetch and increment indices
                         indices = self.indices
-                        idxvals = [builder.load(cgutils.gep(builder, indices, dim))
+                        idxvals = [builder.load(cgutils.gep_inbounds(builder, indices, dim))
                                    for dim in range(ndim)]
                         idxtuple = cgutils.pack_array(builder, idxvals)
                         result.yield_(
@@ -1050,8 +1338,10 @@ def _make_flattening_iter_cls(flatiterty, kind):
 
                     index = builder.add(index, one)
                     builder.store(index, self.index)
-                    ptr = cgutils.pointer_add(builder, ptr, self.stride)
-                    builder.store(ptr, self.pointer)
+
+            def getitem(self, context, builder, arrty, arr, index):
+                ptr = builder.gep(arr.data, [index])
+                return builder.load(ptr)
 
         return CContiguousFlatIter
 
@@ -1081,8 +1371,8 @@ def _make_flattening_iter_cls(flatiterty, kind):
 
                 # Initialize indices and pointers with their start values.
                 for dim in range(ndim):
-                    idxptr = cgutils.gep(builder, indices, dim)
-                    ptrptr = cgutils.gep(builder, pointers, dim)
+                    idxptr = cgutils.gep_inbounds(builder, indices, dim)
+                    ptrptr = cgutils.gep_inbounds(builder, pointers, dim)
                     builder.store(data, ptrptr)
                     builder.store(zero, idxptr)
                     # 0-sized dimensions really indicate an empty array,
@@ -1108,7 +1398,7 @@ def _make_flattening_iter_cls(flatiterty, kind):
                 zero = context.get_constant(types.intp, 0)
                 one = context.get_constant(types.intp, 1)
 
-                bbend = cgutils.append_basic_block(builder, 'end')
+                bbend = builder.append_basic_block('end')
 
                 # Catch already computed iterator exhaustion
                 is_exhausted = cgutils.as_bool_bit(
@@ -1119,14 +1409,14 @@ def _make_flattening_iter_cls(flatiterty, kind):
                 result.set_valid(True)
 
                 # Current pointer inside last dimension
-                last_ptr = cgutils.gep(builder, pointers, ndim - 1)
+                last_ptr = cgutils.gep_inbounds(builder, pointers, ndim - 1)
                 ptr = builder.load(last_ptr)
                 value = context.unpack_value(builder, arrty.dtype, ptr)
                 if kind == 'flat':
                     result.yield_(value)
                 else:
                     # ndenumerate() => yield (indices, value)
-                    idxvals = [builder.load(cgutils.gep(builder, indices, dim))
+                    idxvals = [builder.load(cgutils.gep_inbounds(builder, indices, dim))
                                for dim in range(ndim)]
                     idxtuple = cgutils.pack_array(builder, idxvals)
                     result.yield_(
@@ -1135,7 +1425,7 @@ def _make_flattening_iter_cls(flatiterty, kind):
                 # Update indices and pointers by walking from inner
                 # dimension to outer.
                 for dim in reversed(range(ndim)):
-                    idxptr = cgutils.gep(builder, indices, dim)
+                    idxptr = cgutils.gep_inbounds(builder, indices, dim)
                     idx = builder.add(builder.load(idxptr), one)
 
                     count = shapes[dim]
@@ -1144,13 +1434,13 @@ def _make_flattening_iter_cls(flatiterty, kind):
                     with cgutils.if_likely(builder, in_bounds):
                         # Index is valid => pointer can simply be incremented.
                         builder.store(idx, idxptr)
-                        ptrptr = cgutils.gep(builder, pointers, dim)
+                        ptrptr = cgutils.gep_inbounds(builder, pointers, dim)
                         ptr = builder.load(ptrptr)
                         ptr = cgutils.pointer_add(builder, ptr, stride)
                         builder.store(ptr, ptrptr)
                         # Reset pointers in inner dimensions
                         for inner_dim in range(dim + 1, ndim):
-                            ptrptr = cgutils.gep(builder, pointers, inner_dim)
+                            ptrptr = cgutils.gep_inbounds(builder, pointers, inner_dim)
                             builder.store(ptr, ptrptr)
                         builder.branch(bbend)
                     # Reset index and continue with next dimension
@@ -1162,6 +1452,22 @@ def _make_flattening_iter_cls(flatiterty, kind):
 
                 builder.position_at_end(bbend)
 
+            def getitem(self, context, builder, arrty, arr, index):
+                ndim = arrty.ndim
+                shapes = cgutils.unpack_tuple(builder, arr.shape, count=ndim)
+                strides = cgutils.unpack_tuple(builder, arr.strides, count=ndim)
+
+                # First convert the flattened index into a regular n-dim index
+                indices = []
+                for dim in reversed(range(ndim)):
+                    indices.append(builder.urem(index, shapes[dim]))
+                    index = builder.udiv(index, shapes[dim])
+                indices.reverse()
+
+                ptr = cgutils.get_item_pointer2(builder, arr.data, shapes,
+                                                strides, arrty.layout, indices)
+                return builder.load(ptr)
+
         return FlatIter
 
 
@@ -1199,6 +1505,22 @@ def iternext_numpy_flatiter(context, builder, sig, args, result):
     flatiter.iternext_specific(context, builder, arrty, arr, result)
 
 
+@builtin
+@implement('getitem', types.Kind(types.NumpyFlatType), types.Kind(types.Integer))
+def iternext_numpy_getitem(context, builder, sig, args):
+    flatiterty = sig.args[0]
+    flatiter, index = args
+
+    flatitercls = make_array_flat_cls(flatiterty)
+    flatiter = flatitercls(context, builder, value=flatiter)
+
+    arrty = flatiterty.array_type
+    arrcls = context.make_array(arrty)
+    arr = arrcls(context, builder, value=builder.load(flatiter.array))
+
+    return flatiter.getitem(context, builder, arrty, arr, index)
+
+
 @builtin
 @implement(numpy.ndenumerate, types.Kind(types.Array))
 def make_array_ndenumerate(context, builder, sig, args):
@@ -1249,16 +1571,19 @@ def make_array_ndindex(context, builder, sig, args):
     return nditer._getvalue()
 
 @builtin
-@implement(numpy.ndindex, types.Kind(types.UniTuple))
+@implement(numpy.ndindex, types.Kind(types.BaseTuple))
 def make_array_ndindex(context, builder, sig, args):
     """ndindex(shape)"""
     ndim = sig.return_type.ndim
-    idxty = sig.args[0].dtype
-    tup = args[0]
+    if ndim > 0:
+        idxty = sig.args[0].dtype
+        tup = args[0]
 
-    shape = cgutils.unpack_tuple(builder, tup, ndim)
-    shape = [context.cast(builder, idx, idxty, types.intp)
-             for idx in shape]
+        shape = cgutils.unpack_tuple(builder, tup, ndim)
+        shape = [context.cast(builder, idx, idxty, types.intp)
+                 for idx in shape]
+    else:
+        shape = []
 
     nditercls = make_ndindex_cls(types.NumpyNdIndexType(len(shape)))
     nditer = nditercls(context, builder)
@@ -1300,7 +1625,9 @@ def _empty_nd_impl(context, builder, arrtype, shapes):
     for s in shapes:
         arrlen = builder.mul(arrlen, s)
 
-    if arrtype.layout == 'C':
+    if arrtype.ndim == 0:
+        strides = ()
+    elif arrtype.layout == 'C':
         strides = [itemsize]
         for dimension_size in reversed(shapes[1:]):
             strides.append(builder.mul(strides[-1], dimension_size))
@@ -1315,14 +1642,21 @@ def _empty_nd_impl(context, builder, arrtype, shapes):
             "Don't know how to allocate array with layout '{0}'.".format(
                 arrtype.layout))
 
-    meminfo = context.nrt_meminfo_alloc(builder,
-                                        size=builder.mul(itemsize, arrlen))
+    allocsize = builder.mul(itemsize, arrlen)
+    # NOTE: AVX prefer 32-byte alignment
+    meminfo = context.nrt_meminfo_alloc_aligned(builder, size=allocsize,
+                                                align=32)
+
     data = context.nrt_meminfo_data(builder, meminfo)
 
+    intp_t = context.get_value_type(types.intp)
+    shape_array = cgutils.pack_array(builder, shapes, ty=intp_t)
+    strides_array = cgutils.pack_array(builder, strides, ty=intp_t)
+
     populate_array(ary,
                    data=builder.bitcast(data, datatype.as_pointer()),
-                   shape=cgutils.pack_array(builder, shapes),
-                   strides=cgutils.pack_array(builder, strides),
+                   shape=shape_array,
+                   strides=strides_array,
                    itemsize=itemsize,
                    meminfo=meminfo)
 
@@ -1588,7 +1922,7 @@ def numpy_eye(context, builder, sig, args):
 @builtin
 @implement(numpy.arange, types.Kind(types.Number))
 def numpy_arange_1(context, builder, sig, args):
-    dtype = getattr(numpy, str(sig.return_type.dtype))
+    dtype = as_dtype(sig.return_type.dtype)
 
     def arange(stop):
         return numpy.arange(0, stop, 1, dtype)
@@ -1598,7 +1932,7 @@ def numpy_arange_1(context, builder, sig, args):
 @builtin
 @implement(numpy.arange, types.Kind(types.Number), types.Kind(types.Number))
 def numpy_arange_2(context, builder, sig, args):
-    dtype = getattr(numpy, str(sig.return_type.dtype))
+    dtype = as_dtype(sig.return_type.dtype)
 
     def arange(start, stop):
         return numpy.arange(start, stop, 1, dtype)
@@ -1610,7 +1944,7 @@ def numpy_arange_2(context, builder, sig, args):
 @implement(numpy.arange, types.Kind(types.Number), types.Kind(types.Number),
            types.Kind(types.Number))
 def numpy_arange_3(context, builder, sig, args):
-    dtype = getattr(numpy, str(sig.return_type.dtype))
+    dtype = as_dtype(sig.return_type.dtype)
 
     def arange(start, stop, step):
         return numpy.arange(start, stop, step, dtype)
@@ -1662,7 +1996,7 @@ def numpy_linspace_2(context, builder, sig, args):
 @implement(numpy.linspace, types.Kind(types.Number), types.Kind(types.Number),
            types.Kind(types.Integer))
 def numpy_linspace_3(context, builder, sig, args):
-    dtype = getattr(numpy, str(sig.return_type.dtype))
+    dtype = as_dtype(sig.return_type.dtype)
 
     def linspace(start, stop, num):
         arr = numpy.empty(num, dtype)
@@ -1675,3 +2009,92 @@ def numpy_linspace_3(context, builder, sig, args):
 
     return context.compile_internal(builder, linspace, sig, args)
 
+
+@builtin
+@implement("array.copy", types.Kind(types.Array))
+def array_copy(context, builder, sig, args):
+    arytype = sig.args[0]
+    ary = make_array(arytype)(context, builder, value=args[0])
+    shapes = cgutils.unpack_tuple(builder, ary.shape)
+
+    rettype = sig.return_type
+    ret = _empty_nd_impl(context, builder, rettype, shapes)
+
+    src_data = ary.data
+    dest_data = ret.data
+
+    assert rettype.layout == "C"
+    if arytype.layout == "C":
+        # Fast path: memcpy
+        # Compute array length
+        arrlen = context.get_constant(types.intp, 1)
+        for s in shapes:
+            arrlen = builder.mul(arrlen, s)
+        arrlen = builder.mul(arrlen, ary.itemsize)
+
+        pchar = lc.Type.int(8).as_pointer()
+        memcpy = builder.module.declare_intrinsic(
+            'llvm.memcpy', [pchar, pchar, arrlen.type])
+        builder.call(memcpy,
+                     (builder.bitcast(dest_data, pchar),
+                      builder.bitcast(src_data, pchar),
+                      arrlen,
+                      lc.Constant.int(lc.Type.int(32), 0),
+                      lc.Constant.int(lc.Type.int(1), 0),
+                      ))
+
+    else:
+        src_strides = cgutils.unpack_tuple(builder, ary.strides)
+        dest_strides = cgutils.unpack_tuple(builder, ret.strides)
+        intp_t = context.get_value_type(types.intp)
+
+        with cgutils.loop_nest(builder, shapes, intp_t) as indices:
+            src_ptr = cgutils.get_item_pointer2(builder, src_data,
+                                                shapes, src_strides,
+                                                arytype.layout, indices)
+            dest_ptr = cgutils.get_item_pointer2(builder, dest_data,
+                                                 shapes, dest_strides,
+                                                 rettype.layout, indices)
+            builder.store(builder.load(src_ptr), dest_ptr)
+
+    return ret._getvalue()
+
+
+@builtin
+@implement(numpy.frombuffer, types.Kind(types.Buffer))
+@implement(numpy.frombuffer, types.Kind(types.Buffer), types.Kind(types.DTypeSpec))
+def np_frombuffer(context, builder, sig, args):
+    bufty = sig.args[0]
+    aryty = sig.return_type
+
+    buf = make_array(bufty)(context, builder, value=args[0])
+    out_ary_ty = make_array(aryty)
+    out_ary = out_ary_ty(context, builder)
+    out_datamodel = out_ary._datamodel
+
+    itemsize = get_itemsize(context, aryty)
+    ll_itemsize = lc.Constant.int(buf.itemsize.type, itemsize)
+    nbytes = builder.mul(buf.nitems, buf.itemsize)
+
+    # Check that the buffer size is compatible
+    rem = builder.srem(nbytes, ll_itemsize)
+    is_incompatible = cgutils.is_not_null(builder, rem)
+    with builder.if_then(is_incompatible, likely=False):
+        msg = "buffer size must be a multiple of element size"
+        context.call_conv.return_user_exc(builder, ValueError, (msg,))
+
+    shape = cgutils.pack_array(builder, [builder.sdiv(nbytes, ll_itemsize)])
+    strides = cgutils.pack_array(builder, [ll_itemsize])
+    data = builder.bitcast(buf.data,
+                           context.get_value_type(out_datamodel.get_type('data')))
+
+    populate_array(out_ary,
+                   data=data,
+                   shape=shape,
+                   strides=strides,
+                   itemsize=ll_itemsize,
+                   meminfo=buf.meminfo,
+                   parent=buf.parent,)
+
+    return out_ary._getvalue()
+
