@@ -152,6 +152,9 @@ class BaseContext(object):
     # NRT
     enable_nrt = False
 
+    # PYCC
+    aot_mode = False
+
     def __init__(self, typing_context):
         _load_global_helpers()
         self.address_size = utils.MACHINE_BITS
@@ -397,6 +400,9 @@ class BaseContext(object):
         elif ty in types.real_domain:
             return Constant.real(lty, val)
 
+        elif isinstance(ty, (types.NPDatetime, types.NPTimedelta)):
+            return Constant.real(lty, val.astype(numpy.int64))
+
         elif isinstance(ty, types.UniTuple):
             consts = [self.get_constant(ty.dtype, v) for v in val]
             return Constant.array(consts[0].type, consts)
@@ -795,7 +801,7 @@ class BaseContext(object):
         for obj in pyargs:
             api.decref(obj)
 
-        with cgutils.ifthen(builder, cgutils.is_null(builder, res)):
+        with builder.if_then(cgutils.is_null(builder, res)):
             self.call_conv.return_exc(builder)
 
         if retty == types.none:
@@ -815,7 +821,7 @@ class BaseContext(object):
         return builder.call(puts, [text])
 
     def debug_print(self, builder, text):
-        mod = cgutils.get_module(builder)
+        mod = builder.module
         cstr = self.insert_const_string(mod, str(text))
         self.print_string(builder, cstr)
 
@@ -855,30 +861,39 @@ class BaseContext(object):
         codegen.add_linking_library(cres.library)
         return cres
 
-    def compile_internal(self, builder, impl, sig, args, locals={}):
-        """Invoke compiler to implement a function for a nopython function
+    def compile_subroutine(self, builder, impl, sig, locals={}):
+        """
+        Compile the function *impl* for the given *sig* (in nopython mode).
+        Return a placeholder object that's callable from another Numba
+        function.
         """
         cache_key = (impl.__code__, sig)
         if impl.__closure__:
             # XXX This obviously won't work if a cell's value is
             # unhashable.
             cache_key += tuple(c.cell_contents for c in impl.__closure__)
-        fndesc = self.cached_internal_func.get(cache_key)
-
-        if fndesc is None:
+        ty = self.cached_internal_func.get(cache_key)
+        if ty is None:
             cres = self.compile_only_no_cache(builder, impl, sig,
                                               locals=locals)
-            fndesc = cres.fndesc
-            self.cached_internal_func[cache_key] = fndesc
+            ty = types.NumbaFunction(cres.fndesc, sig)
+            self.cached_internal_func[cache_key] = ty
+        return ty
 
-        return self.call_internal(builder, fndesc, sig, args)
+    def compile_internal(self, builder, impl, sig, args, locals={}):
+        """
+        Like compile_subroutine(), but also call the function with the given
+        *args*.
+        """
+        ty = self.compile_subroutine(builder, impl, sig, locals)
+        return self.call_internal(builder, ty.fndesc, sig, args)
 
     def call_internal(self, builder, fndesc, sig, args):
         """Given the function descriptor of an internally compiled function,
         emit a call to that function with the given arguments.
         """
         # Add call to the generated function
-        llvm_mod = cgutils.get_module(builder)
+        llvm_mod = builder.module
         fn = self.declare_function(llvm_mod, fndesc)
         status, res = self.call_conv.call_function(builder, fn, sig.return_type,
                                                    sig.args, args)
@@ -998,56 +1013,71 @@ class BaseContext(object):
     def nrt_meminfo_alloc(self, builder, size):
         if not self.enable_nrt:
             raise Exception("Require NRT")
-        mod = cgutils.get_module(builder)
+        mod = builder.module
         fnty = llvmir.FunctionType(llvmir.IntType(8).as_pointer(),
-            [self.get_value_type(types.intp)])
+                                   [self.get_value_type(types.intp)])
         fn = mod.get_or_insert_function(fnty, name="NRT_MemInfo_alloc_safe")
+        fn.return_value.add_attribute("noalias")
         return builder.call(fn, [size])
 
+    def nrt_meminfo_alloc_aligned(self, builder, size, align):
+        """
+        Allocate a new MemInfo of `size` bytes and and align the data pointer
+        to `align` bytes.  The `align` arg can be either a Python int or a LLVM
+        uint32 value.
+        """
+        if not self.enable_nrt:
+            raise Exception("Require NRT")
+        mod = builder.module
+        intp = self.get_value_type(types.intp)
+        u32 = self.get_value_type(types.uint32)
+        fnty = llvmir.FunctionType(llvmir.IntType(8).as_pointer(), [intp, u32])
+        fn = mod.get_or_insert_function(fnty,
+                                        name="NRT_MemInfo_alloc_safe_aligned")
+        if isinstance(align, int):
+            align = self.get_constant(types.uint32, align)
+        else:
+            assert align.type == u32, "align must be a uint32"
+        return builder.call(fn, [size, align])
+
     def nrt_meminfo_data(self, builder, meminfo):
         if not self.enable_nrt:
             raise Exception("Require NRT")
-        mod = cgutils.get_module(builder)
+        mod = builder.module
         voidptr = llvmir.IntType(8).as_pointer()
         fnty = llvmir.FunctionType(voidptr, [voidptr])
         fn = mod.get_or_insert_function(fnty, name="NRT_MemInfo_data")
+        fn.return_value.add_attribute("noalias")
         return builder.call(fn, [meminfo])
 
     def get_nrt_meminfo(self, builder, typ, value):
         return self.data_model_manager[typ].get_nrt_meminfo(builder, value)
 
-    def nrt_incref(self, builder, typ, value):
+    def _call_nrt_incref_decref(self, builder, typ, value, funcname):
         if not self.enable_nrt:
             raise Exception("Require NRT")
 
         members = self.data_model_manager[typ].traverse(builder, value)
         for mt, mv in members:
-            self.nrt_incref(builder, mt, mv)
+            self._call_nrt_incref_decref(builder, mt, mv, funcname)
 
         meminfo = self.get_nrt_meminfo(builder, typ, value)
         if meminfo:
-            mod = cgutils.get_module(builder)
+            mod = builder.module
             fnty = llvmir.FunctionType(llvmir.VoidType(),
-                [llvmir.IntType(8).as_pointer()])
-            fn = mod.get_or_insert_function(fnty, name="NRT_incref")
-
+                                       [llvmir.IntType(8).as_pointer()])
+            fn = mod.get_or_insert_function(fnty, name=funcname)
+            # XXX "nonnull" causes a crash in test_dyn_array: can this
+            # function be called with a NULL pointer?
+            fn.args[0].add_attribute("noalias")
+            fn.args[0].add_attribute("nocapture")
             builder.call(fn, [meminfo])
 
+    def nrt_incref(self, builder, typ, value):
+        self._call_nrt_incref_decref(builder, typ, value, "NRT_incref")
+
     def nrt_decref(self, builder, typ, value):
-        if not self.enable_nrt:
-            raise Exception("Require NRT")
-
-        members = self.data_model_manager[typ].traverse(builder, value)
-        for mt, mv in members:
-            self.nrt_decref(builder, mt, mv)
-
-        meminfo = self.get_nrt_meminfo(builder, typ, value)
-        if meminfo:
-            mod = cgutils.get_module(builder)
-            fnty = llvmir.FunctionType(llvmir.VoidType(),
-                [llvmir.IntType(8).as_pointer()])
-            fn = mod.get_or_insert_function(fnty, name="NRT_decref")
-            builder.call(fn, [meminfo])
+        self._call_nrt_incref_decref(builder, typ, value, "NRT_decref")
 
 
 class _wrap_impl(object):
