@@ -78,6 +78,10 @@ class CPUContext(BaseContext):
         From the pointer *clo* to a _dynfunc.Closure, get a pointer
         to the enclosed _dynfunc.Environment.
         """
+        with cgutils.if_unlikely(builder, cgutils.is_null(builder, clo)):
+            self.debug_print(builder, "Fatal error: missing _dynfunc.Closure")
+            builder.unreachable()
+
         clo_body_ptr = cgutils.pointer_add(
             builder, clo, _dynfunc._impl_info['offsetof_closure_body'])
         clo_body = ClosureBody(self, builder, ref=clo_body_ptr, cast_ref=True)
@@ -115,13 +119,13 @@ class CPUContext(BaseContext):
 
         install_atomic_refct(mod)
 
-    def create_cpython_wrapper(self, library, fndesc, call_helper,
+    def create_cpython_wrapper(self, library, fndesc, env, call_helper,
                                release_gil=False):
         wrapper_module = self.create_module("wrapper")
         fnty = self.call_conv.get_function_type(fndesc.restype, fndesc.argtypes)
         wrapper_callee = wrapper_module.add_function(fnty, fndesc.llvm_func_name)
         builder = PyCallWrapper(self, wrapper_module, wrapper_callee,
-                                fndesc, call_helper=call_helper,
+                                fndesc, env, call_helper=call_helper,
                                 release_gil=release_gil)
         builder.build()
         library.add_ir_module(wrapper_module)
