@@ -9,7 +9,6 @@ from numba.tests.support import TestCase
 import numba.unittest_support as unittest
 from numba import testing
 
-
 enable_pyobj_flags = Flags()
 enable_pyobj_flags.set("enable_pyobject")
 
@@ -25,6 +24,7 @@ def make_consumer(gen_func):
         for y in gen_func(x):
             res += y
         return res
+
     return consumer
 
 
@@ -32,18 +32,21 @@ def gen1(x):
     for i in range(x):
         yield i
 
+
 def gen2(x):
     for i in range(x):
         yield i
         for j in range(1, 3):
             yield i + j
 
+
 def gen3(x):
     # Polymorphic yield types must be unified
     yield x
     yield x + 1.5
     yield x + 1j
 
+
 def gen4(x, y, z):
     for i in range(3):
         yield z
@@ -51,6 +54,7 @@ def gen4(x, y, z):
     return
     yield x
 
+
 def gen5():
     # The bytecode for this generator doesn't contain any YIELD_VALUE
     # (it's optimized away).  We fail typing it, since the yield type
@@ -58,6 +62,7 @@ def gen5():
     if 0:
         yield 1
 
+
 def gen6(a, b):
     # Infinite loop: exercise computation of state variables
     x = a + 1
@@ -65,6 +70,7 @@ def gen6(a, b):
         y = b + 2
         yield x + y
 
+
 def gen7(arr):
     # Array variable in generator state
     for i in range(arr.size):
@@ -77,24 +83,25 @@ def genobj(x):
 
 
 def return_generator_expr(x):
-    return (i*2 for i in x)
+    return (i * 2 for i in x)
 
 
 def gen_ndindex(shape):
     for ind in np.ndindex(shape):
         yield ind
 
+
 def gen_flat(arr):
     for val in arr.flat:
         yield val
 
+
 def gen_ndenumerate(arr):
     for tup in np.ndenumerate(arr):
         yield tup
 
 
 class TestGenerators(TestCase):
-
     def check_generator(self, pygen, cgen):
         self.assertEqual(next(cgen), next(pygen))
         # Use list comprehensions to make sure we trash the generator's
@@ -258,7 +265,6 @@ class TestGenerators(TestCase):
 
 
 class TestGenExprs(TestCase):
-
     @testing.allow_interpreter_mode
     def test_return_generator_expr(self):
         pyfunc = return_generator_expr
@@ -267,11 +273,11 @@ class TestGenExprs(TestCase):
         self.assertEqual(sum(cfunc([1, 2, 3])), sum(pyfunc([1, 2, 3])))
 
 
-
 def nrt_gen0(ary):
     for elem in ary:
         yield elem
 
+
 def nrt_gen1(ary1, ary2):
     for e1, e2 in zip(ary1, ary2):
         yield e1
@@ -295,7 +301,6 @@ class TestNrtArrayGen(TestCase):
         self.assertEqual(sys.getrefcount(py_ary),
                          sys.getrefcount(c_ary))
 
-
     def test_nrt_gen1(self):
         pygen = nrt_gen1
         cgen = jit(nopython=True)(pygen)
@@ -316,7 +321,7 @@ class TestNrtArrayGen(TestCase):
         self.assertEqual(sys.getrefcount(py_ary1),
                          sys.getrefcount(c_ary1))
         self.assertEqual(sys.getrefcount(py_ary2),
-                          sys.getrefcount(c_ary2))
+                         sys.getrefcount(c_ary2))
 
     def test_combine_gen0_gen1(self):
         """
@@ -382,6 +387,167 @@ class TestNrtArrayGen(TestCase):
         self.assertEqual(sys.getrefcount(py_ary),
                          sys.getrefcount(c_ary))
 
+    def test_nrt_nested_gen(self):
+
+        def gen0(arr):
+            for i in range(arr.size):
+                yield arr
+
+        def factory(gen0):
+            def gen1(arr):
+                out = np.zeros_like(arr)
+                for x in gen0(arr):
+                    out = out + x
+                return out, arr
+
+            return gen1
+
+        py_arr = np.arange(10)
+        c_arr = py_arr.copy()
+        py_res, py_old = factory(gen0)(py_arr)
+        c_gen = jit(nopython=True)(factory(jit(nopython=True)(gen0)))
+        c_res, c_old = c_gen(c_arr)
+
+        self.assertIsNot(py_arr, c_arr)
+        self.assertIs(py_old, py_arr)
+        self.assertIs(c_old, c_arr)
+
+        np.testing.assert_equal(py_res, c_res)
+
+        self.assertEqual(sys.getrefcount(py_res),
+                         sys.getrefcount(c_res))
+
+        # The below test will fail due to generator finalizer not invoked.
+        # This kept a reference of the c_old.
+        #
+        # self.assertEqual(sys.getrefcount(py_old),
+        #                  sys.getrefcount(c_old))
+
+    @unittest.expectedFailure
+    def test_nrt_nested_gen_refct(self):
+        def gen0(arr):
+            yield arr
+
+        def factory(gen0):
+            def gen1(arr):
+                for out in gen0(arr):
+                    return out
+
+            return gen1
+
+        py_arr = np.arange(10)
+        c_arr = py_arr.copy()
+        py_old = factory(gen0)(py_arr)
+        c_gen = jit(nopython=True)(factory(jit(nopython=True)(gen0)))
+        c_old = c_gen(c_arr)
+
+        self.assertIsNot(py_arr, c_arr)
+        self.assertIs(py_old, py_arr)
+        self.assertIs(c_old, c_arr)
+
+        self.assertEqual(sys.getrefcount(py_old),
+                         sys.getrefcount(c_old))
+
+    def test_nrt_nested_nopython_gen(self):
+        """
+        Test nesting three generators
+        """
+
+        def factory(decor=lambda x: x):
+            @decor
+            def foo(a, n):
+                for i in range(n):
+                    yield a[i]
+                    a[i] += i
+
+            @decor
+            def bar(n):
+                a = np.arange(n)
+                for i in foo(a, n):
+                    yield i * 2
+                for i in range(a.size):
+                    yield a[i]
+
+            @decor
+            def cat(n):
+                for i in bar(n):
+                    yield i + i
+
+            return cat
+
+        py_gen = factory()
+        c_gen = factory(jit(nopython=True))
+
+        py_res = list(py_gen(10))
+        c_res = list(c_gen(10))
+
+        self.assertEqual(py_res, c_res)
+
+
+
+class TestGeneratorWithNRT(TestCase):
+    def test_issue_1254(self):
+        """
+        Missing environment for returning array
+        """
+
+        @jit(nopython=True)
+        def random_directions(n):
+            for i in range(n):
+                vec = np.empty(3)
+                vec[:] = 12
+                yield vec
+
+        outputs = list(random_directions(5))
+        self.assertEqual(len(outputs), 5)
+
+        expect = np.empty(3)
+        expect[:] = 12
+        for got in outputs:
+            np.testing.assert_equal(expect, got)
+
+    def test_issue_1265(self):
+        """
+        Double-free for locally allocated, non escaping NRT objects
+        """
+        def py_gen(rmin, rmax, nr):
+            a = np.linspace(rmin, rmax, nr)
+            yield a[0]
+            yield a[1]
+
+        c_gen = jit(nopython=True)(py_gen)
+
+        py_res = list(py_gen(-2, 2, 100))
+        c_res = list(c_gen(-2, 2, 100))
+
+        self.assertEqual(py_res, c_res)
+
+        def py_driver(args):
+            rmin, rmax, nr = args
+            points = np.empty(nr, dtype=np.complex128)
+            for i, c in enumerate(py_gen(rmin, rmax, nr)):
+                points[i] = c
+
+            return points
+
+        @jit(nopython=True)
+        def c_driver(args):
+            rmin, rmax, nr = args
+            points = np.empty(nr, dtype=np.complex128)
+            for i, c in enumerate(c_gen(rmin, rmax, nr)):
+                points[i] = c
+
+            return points
+
+        n = 2
+        patches = (-2, -1, n)
+
+        py_res = py_driver(patches)
+        # The error will cause a segfault here
+        c_res = c_driver(patches)
+
+        np.testing.assert_equal(py_res, c_res)
+
 
 if __name__ == '__main__':
     unittest.main()
