@@ -19,16 +19,7 @@ import itertools
 
 from numba import ir, types, utils, config, six
 from numba.utils import RANGE_ITER_OBJECTS
-
-
-class TypingError(Exception):
-    def __init__(self, msg, loc=None):
-        self.msg = msg
-        self.loc = loc
-        if loc:
-            super(TypingError, self).__init__("%s\n%s" % (msg, loc.strformat()))
-        else:
-            super(TypingError, self).__init__("%s" % (msg,))
+from .errors import TypingError
 
 
 class TypeVar(object):
@@ -53,7 +44,7 @@ class TypeVar(object):
             if set(types) != self.typeset:
                 [expect] = list(self.typeset)
                 for ty in types:
-                    if self.context.type_compatibility(ty, expect) is None:
+                    if self.context.can_convert(ty, expect) is None:
                         raise TypingError("No conversion from %s to %s for "
                                           "'%s'" % (ty, expect, self.var))
         else:
@@ -65,7 +56,7 @@ class TypeVar(object):
     def lock(self, typ):
         if self.locked:
             [expect] = list(self.typeset)
-            if self.context.type_compatibility(typ, expect) is None:
+            if self.context.can_convert(typ, expect) is None:
                 raise TypingError("No conversion from %s to %s for "
                                   "'%s'" % (typ, expect, self.var))
         else:
@@ -419,8 +410,8 @@ class TypeInferer(object):
                 raise TypeError("Variable %s has no type" % var)
             else:
                 unified = self.context.unify_types(*tv.get())
-            if unified == types.pyobject:
-                raise TypingError("Var '%s' unified to object: %s" % (var, tv))
+            if unified is types.pyobject:
+                raise TypingError("Can't unify types of variable '%s': %s" % (var, tv))
             typdict[var] = unified
         retty = self.get_return_type(typdict)
         fntys = self.get_function_types(typdict)
@@ -493,16 +484,12 @@ class TypeInferer(object):
             if isinstance(term, ir.Return):
                 rettypes.add(typemap[term.value.name])
 
-        if types.none in rettypes:
-            # Special case None return
-            rettypes = rettypes - set([types.none])
-            if rettypes:
-                unified = self.context.unify_types(*rettypes)
-                return types.Optional(unified)
-            else:
-                return types.none
-        elif rettypes:
+        if rettypes:
             unified = self.context.unify_types(*rettypes)
+            if unified is types.pyobject:
+                raise TypingError("Can't unify return type from the "
+                                  "following types: %s"
+                                  % ", ".join(sorted(map(str, rettypes))))
             return unified
         else:
             return types.none
