@@ -4,154 +4,15 @@ the targets to choose their representation.
 """
 from __future__ import print_function, division, absolute_import
 
-import itertools
 import struct
 import weakref
 
 import numpy
 
-from .six import add_metaclass
+# All abstract types are exposed through this module
+from .abstracttypes import *
 from . import npdatetime, utils
-
-
-# Types are added to a global registry (_typecache) in order to assign
-# them unique integer codes for fast matching in _dispatcher.c.
-# However, we also want types to be disposable, therefore we ensure
-# each type is interned as a weak reference, so that it lives only as
-# long as necessary to keep a stable type code.
-_typecodes = itertools.count()
-
-def _autoincr():
-    n = next(_typecodes)
-    # 4 billion types should be enough, right?
-    assert n < 2 ** 32, "Limited to 4 billion types"
-    return n
-
-_typecache = {}
-
-def _on_type_disposal(wr, _pop=_typecache.pop):
-    _pop(wr, None)
-
-
-class _TypeMetaclass(type):
-    """
-    A metaclass that will intern instances after they are created.
-    This is done by first creating a new instance (including calling
-    __init__, which sets up the required attributes for equality
-    and hashing), then looking it up in the _typecache registry.
-    """
-
-    def __call__(cls, *args, **kwargs):
-        """
-        Instantiate *cls* (a Type subclass, presumably) and intern it.
-        If an interned instance already exists, it is returned, otherwise
-        the new instance is returned.
-        """
-        inst = type.__call__(cls, *args, **kwargs)
-        # Try to intern the created instance
-        wr = weakref.ref(inst, _on_type_disposal)
-        orig = _typecache.get(wr)
-        orig = orig and orig()
-        if orig is not None:
-            return orig
-        else:
-            inst._code = _autoincr()
-            _typecache[wr] = wr
-            inst.post_init()
-            return inst
-
-
-@add_metaclass(_TypeMetaclass)
-class Type(object):
-    """
-    The default behavior is to provide equality through `name` attribute.
-    Two types are equal if there `name` are equal.
-    Subclass can refine this behavior.
-    """
-
-    mutable = False
-
-    def __init__(self, name, param=False):
-        self.name = name
-        self.is_parametric = param
-
-    def post_init(self):
-        """
-        A method called when the instance is fully initialized and has
-        a registered typecode in its _code attribute.  Does nothing by
-        default, but can be overriden.
-        """
-
-    @property
-    def key(self):
-        """
-        A property used for __eq__, __ne__ and __hash__.  Can be overriden
-        in subclasses.
-        """
-        return self.name
-
-    def __repr__(self):
-        return self.name
-
-    def __hash__(self):
-        return hash(self.key)
-
-    def __eq__(self, other):
-        return self.__class__ is other.__class__ and self.key == other.key
-
-    def __ne__(self, other):
-        return not (self == other)
-
-    def __call__(self, *args):
-        if len(args) == 1 and not isinstance(args[0], Type):
-            return self.cast_python_value(args[0])
-        return Prototype(args=args, return_type=self)
-
-    def __getitem__(self, args):
-        assert not isinstance(self, Array)
-        ndim, layout = self._determine_array_spec(args)
-        return Array(dtype=self, ndim=ndim, layout=layout)
-
-    def _determine_array_spec(self, args):
-        if isinstance(args, (tuple, list)):
-            ndim = len(args)
-            if args[0].step == 1:
-                layout = 'F'
-            elif args[-1].step == 1:
-                layout = 'C'
-            else:
-                layout = 'A'
-        elif isinstance(args, slice):
-            ndim = 1
-            if args.step == 1:
-                layout = 'C'
-            else:
-                layout = 'A'
-        else:
-            ndim = 1
-            layout = 'A'
-
-        return ndim, layout
-
-
-    __iter__ = NotImplemented
-    cast_python_value = NotImplemented
-
-    def coerce(self, typingctx, other):
-        """Override this method to implement specialized coercion logic
-        for extending unify_pairs().  Only use this if the coercion logic cannot
-        be expressed as simple casting rules.
-        """
-        return NotImplemented
-
-
-class OpaqueType(Type):
-    """
-    To deal with externally defined literal types
-    """
-
-    def __init__(self, name):
-        super(OpaqueType, self).__init__(name)
+from .typeconv import Conversion
 
 
 class Boolean(Type):
@@ -160,12 +21,6 @@ class Boolean(Type):
         return bool(value)
 
 
-class Number(Type):
-    """
-    Base class for number types.
-    """
-
-
 @utils.total_ordering
 class Integer(Number):
     def __init__(self, *args, **kws):
@@ -254,35 +109,29 @@ class NPDatetime(_NPDatetimeBase):
     type_name = 'datetime64'
 
 
-class Prototype(Type):
-    def __init__(self, args, return_type):
-        self.args = args
-        self.return_type = return_type
-        name = "%s(%s)" % (return_type, ', '.join(str(a) for a in args))
-        super(Prototype, self).__init__(name=name)
-
-
-class Dummy(Type):
-    """
-    For type that does not really have a representation and is compatible
-    with a void*.
-    """
-    pass
-
-
 class Phantom(Dummy):
     """
     A type that cannot be materialized.  A Phantom cannot be used as
     argument or return type.
     """
-    pass
 
 
 class Opaque(Dummy):
     """
     A type that is a opaque pointer.
     """
-    pass
+
+
+class PyObject(Dummy):
+    """
+    A generic CPython object.
+    """
+
+
+class RawPointer(Dummy):
+    """
+    A raw pointer without any specific meaning.
+    """
 
 
 class Kind(Type):
@@ -332,19 +181,6 @@ class Macro(Type):
         return self.template
 
 
-class Callable(Type):
-    """
-    Base class for callables.
-    """
-
-    def get_call_type(self, context, args, kws):
-        """
-        Using the typing *context*, resolve the callable's signature for
-        the given arguments.  A signature object is returned, or None.
-        """
-        raise NotImplementedError
-
-
 class Function(Callable, Opaque):
     def __init__(self, template):
         self.template = template
@@ -359,18 +195,7 @@ class Function(Callable, Opaque):
         return self.template(context).apply(args, kws)
 
 
-class DTypeSpec(Opaque):
-    """
-    Base class for types usable as "dtype" arguments to various Numpy APIs
-    (e.g. np.empty()).
-    """
-
-    @property
-    def dtype(self):
-        raise NotImplementedError
-
-
-class NumberClass(Callable, DTypeSpec):
+class NumberClass(Callable, DTypeSpec, Opaque):
     """
     Type class for number classes (e.g. "np.float64").
     """
@@ -389,7 +214,7 @@ class NumberClass(Callable, DTypeSpec):
         return self.instance_type
 
 
-class DType(DTypeSpec):
+class DType(DTypeSpec, Opaque):
     """
     Type class for Numpy dtypes.
     """
@@ -502,6 +327,7 @@ class ExternalFunctionPointer(Function):
 class ExternalFunction(Function):
     """
     A named native function (resolvable by LLVM).
+    For internal use only.
     """
 
     def __init__(self, symbol, sig):
@@ -511,29 +337,55 @@ class ExternalFunction(Function):
         template = typing.make_concrete_template(symbol, symbol, [sig])
         super(ExternalFunction, self).__init__(template)
 
+    @property
+    def key(self):
+        return self.symbol, self.sig
+
+
+class NumbaFunction(Function):
+    """
+    A named native function with the Numba calling convention
+    (resolvable by LLVM).
+    For internal use only.
+    """
+
+    def __init__(self, fndesc, sig):
+        from . import typing
+        self.fndesc = fndesc
+        self.sig = sig
+        template = typing.make_concrete_template(fndesc.qualname,
+                                                 fndesc.qualname, [sig])
+        super(NumbaFunction, self).__init__(template)
+
+    @property
+    def key(self):
+        return self.fndesc.unique_name, self.sig
+
 
 class BoundFunction(Function):
     def __init__(self, template, this):
         self.this = this
+        # Create a derived template with an attribute *this*
         newcls = type(template.__name__ + '.' + str(this), (template,),
                       dict(this=this))
         super(BoundFunction, self).__init__(newcls)
 
     @property
     def key(self):
-        return (self.template.__name__, self.this)
+        return self.template.key, self.this
 
 
 class Method(Function):
     def __init__(self, template, this):
         self.this = this
+        # Create a derived template with an attribute *this*
         newcls = type(template.__name__ + '.' + str(this), (template,),
                       dict(this=this))
         super(Method, self).__init__(newcls)
 
     @property
     def key(self):
-        return (self.template.__name__, self.this)
+        return self.template.key, self.this
 
 
 class Pair(Type):
@@ -552,37 +404,27 @@ class Pair(Type):
         return self.first_type, self.second_type
 
 
-class IterableType(Type):
-    """
-    Base class for iterable types.
-    Derived classes should implement the *iterator_type* attribute.
-    """
-
-
 class SimpleIterableType(IterableType):
 
     def __init__(self, name, iterator_type):
-        self.iterator_type = iterator_type
+        self._iterator_type = iterator_type
         super(SimpleIterableType, self).__init__(name, param=True)
 
-
-class IteratorType(IterableType):
-    """
-    Base class for all iterator types.
-    Derived classes should implement the *yield_type* attribute.
-    """
-
-    def __init__(self, name, **kwargs):
-        self.iterator_type = self
-        super(IteratorType, self).__init__(name, **kwargs)
+    @property
+    def iterator_type(self):
+        return self._iterator_type
 
 
 class SimpleIteratorType(IteratorType):
 
     def __init__(self, name, yield_type):
-        self.yield_type = yield_type
+        self._yield_type = yield_type
         super(SimpleIteratorType, self).__init__(name, param=True)
 
+    @property
+    def yield_type(self):
+        return self._yield_type
+
 
 class RangeType(SimpleIterableType):
     pass
@@ -592,7 +434,7 @@ class RangeIteratorType(SimpleIteratorType):
     pass
 
 
-class Generator(IteratorType):
+class Generator(SimpleIteratorType):
     """
     Type class for Numba-compiled generator objects.
     """
@@ -602,35 +444,34 @@ class Generator(IteratorType):
         self.gen_func = gen_func
         self.arg_types = tuple(arg_types)
         self.state_types = tuple(state_types)
-        self.yield_type = yield_type
         self.has_finalizer = has_finalizer
         name = "%s generator(func=%s, args=%s, has_finalizer=%s)" % (
-            self.yield_type, self.gen_func, self.arg_types,
+            yield_type, self.gen_func, self.arg_types,
             self.has_finalizer)
-        super(Generator, self).__init__(name, param=True)
+        super(Generator, self).__init__(name, yield_type)
 
     @property
     def key(self):
         return self.gen_func, self.arg_types, self.yield_type, self.has_finalizer
 
 
-class NumpyFlatType(IteratorType):
+class NumpyFlatType(SimpleIteratorType):
     """
     Type class for `ndarray.flat()` objects.
     """
 
     def __init__(self, arrty):
         self.array_type = arrty
-        self.yield_type = arrty.dtype
+        yield_type = arrty.dtype
         name = "array.flat({arrayty})".format(arrayty=arrty)
-        super(NumpyFlatType, self).__init__(name, param=True)
+        super(NumpyFlatType, self).__init__(name, yield_type)
 
     @property
     def key(self):
         return self.array_type
 
 
-class NumpyNdEnumerateType(IteratorType):
+class NumpyNdEnumerateType(SimpleIteratorType):
     """
     Type class for `np.ndenumerate()` objects.
     """
@@ -639,32 +480,32 @@ class NumpyNdEnumerateType(IteratorType):
         self.array_type = arrty
         # XXX making this a uintp has the side effect of forcing some
         # arithmetic operations to return a float result.
-        self.yield_type = Tuple((UniTuple(intp, arrty.ndim), arrty.dtype))
+        yield_type = Tuple((UniTuple(intp, arrty.ndim), arrty.dtype))
         name = "ndenumerate({arrayty})".format(arrayty=arrty)
-        super(NumpyNdEnumerateType, self).__init__(name, param=True)
+        super(NumpyNdEnumerateType, self).__init__(name, yield_type)
 
     @property
     def key(self):
         return self.array_type
 
 
-class NumpyNdIndexType(IteratorType):
+class NumpyNdIndexType(SimpleIteratorType):
     """
     Type class for `np.ndindex()` objects.
     """
 
     def __init__(self, ndim):
         self.ndim = ndim
-        self.yield_type = UniTuple(intp, self.ndim)
+        yield_type = UniTuple(intp, self.ndim)
         name = "ndindex(dims={ndim})".format(ndim=ndim)
-        super(NumpyNdIndexType, self).__init__(name, param=True)
+        super(NumpyNdIndexType, self).__init__(name, yield_type)
 
     @property
     def key(self):
         return self.ndim
 
 
-class EnumerateType(IteratorType):
+class EnumerateType(SimpleIteratorType):
     """
     Type class for `enumerate` objects.
     Type instances are parametered with the underlying source type.
@@ -672,16 +513,16 @@ class EnumerateType(IteratorType):
 
     def __init__(self, iterable_type):
         self.source_type = iterable_type.iterator_type
-        self.yield_type = Tuple([intp, self.source_type.yield_type])
+        yield_type = Tuple([intp, self.source_type.yield_type])
         name = 'enumerate(%s)' % (self.source_type)
-        super(EnumerateType, self).__init__(name, param=True)
+        super(EnumerateType, self).__init__(name, yield_type)
 
     @property
     def key(self):
         return self.source_type
 
 
-class ZipType(IteratorType):
+class ZipType(SimpleIteratorType):
     """
     Type class for `zip` objects.
     Type instances are parametered with the underlying source types.
@@ -689,9 +530,9 @@ class ZipType(IteratorType):
 
     def __init__(self, iterable_types):
         self.source_types = tuple(tp.iterator_type for tp in iterable_types)
-        self.yield_type = Tuple(tp.yield_type for tp in self.source_types)
+        yield_type = Tuple(tp.yield_type for tp in self.source_types)
         name = 'zip(%s)' % ', '.join(str(tp) for tp in self.source_types)
-        super(ZipType, self).__init__(name, param=True)
+        super(ZipType, self).__init__(name, yield_type)
 
     @property
     def key(self):
@@ -754,7 +595,7 @@ class Record(Type):
         return [(f, t) for f, (t, _) in self.fields.items()]
 
 
-class ArrayIterator(IteratorType):
+class ArrayIterator(SimpleIteratorType):
     """
     Type class for iterators of array and buffer objects.
     """
@@ -764,10 +605,10 @@ class ArrayIterator(IteratorType):
         name = "iter(%s)" % (self.array_type,)
         nd = array_type.ndim
         if nd == 0 or nd == 1:
-            self.yield_type = array_type.dtype
+            yield_type = array_type.dtype
         else:
-            self.yield_type = array_type.copy(ndim=array_type.ndim - 1)
-        super(ArrayIterator, self).__init__(name, param=True)
+            yield_type = array_type.copy(ndim=array_type.ndim - 1)
+        super(ArrayIterator, self).__init__(name, yield_type)
 
 
 class Buffer(IterableType):
@@ -797,7 +638,11 @@ class Buffer(IterableType):
                 type_name = "readonly %s" % type_name
             name = "%s(%s, %sd, %s)" % (type_name, dtype, ndim, layout)
         super(Buffer, self).__init__(name, param=True)
-        self.iterator_type = ArrayIterator(self)
+        self._iterator_type = ArrayIterator(self)
+
+    @property
+    def iterator_type(self):
+        return self._iterator_type
 
     def copy(self, dtype=None, ndim=None, layout=None):
         if dtype is None:
@@ -868,16 +713,6 @@ class Array(Buffer):
             name = "%s(%s, %sd, %s)" % (type_name, dtype, ndim, layout)
         super(Array, self).__init__(dtype, ndim, layout, name=name)
 
-    def post_init(self):
-        """
-        Install conversion from this layout (if non-'A') to 'A' layout.
-        """
-        if self.layout != 'A':
-            from numba.typeconv.rules import default_casting_rules as tcr
-            ary_any = self.copy(layout='A')
-            # XXX This will make the types immortal
-            tcr.safe(self, ary_any)
-
     def copy(self, dtype=None, ndim=None, layout=None, readonly=None):
         if dtype is None:
             dtype = self.dtype
@@ -893,6 +728,30 @@ class Array(Buffer):
     def key(self):
         return self.dtype, self.ndim, self.layout, self.mutable
 
+    def unify(self, typingctx, other):
+        """
+        Unify this with the *other* Array.
+        """
+        if (isinstance(other, Array) and other.ndim == self.ndim
+            and other.dtype == self.dtype):
+            if self.layout == other.layout:
+                layout = self.layout
+            else:
+                layout = 'A'
+            readonly = not (self.mutable and other.mutable)
+            return Array(dtype=self.dtype, ndim=self.ndim, layout=layout,
+                         readonly=readonly)
+
+    def can_convert_to(self, typingctx, other):
+        """
+        Convert this Array to the *other*.
+        """
+        if (isinstance(other, Array) and other.ndim == self.ndim
+            and other.dtype == self.dtype):
+            if (other.layout in ('A', self.layout)
+                and (self.mutable or not other.mutable)):
+                return Conversion.safe
+
 
 class ArrayCTypes(Type):
     """
@@ -902,7 +761,7 @@ class ArrayCTypes(Type):
         # This depends on the ndim for the shape and strides attributes,
         # even though they are not implemented, yet.
         self.ndim = arytype.ndim
-        name = "ArrayCType(ndim={0})".format(self.ndim)
+        name = "ArrayCTypes(ndim={0})".format(self.ndim)
         super(ArrayCTypes, self).__init__(name, param=True)
 
     @property
@@ -910,6 +769,20 @@ class ArrayCTypes(Type):
         return self.ndim
 
 
+class ArrayFlags(Type):
+    """
+    This is the type for `numpy.ndarray.flags`.
+    """
+    def __init__(self, arytype):
+        self.array_type = arytype
+        name = "ArrayFlags({0})".format(self.array_type)
+        super(ArrayFlags, self).__init__(name, param=True)
+
+    @property
+    def key(self):
+        return self.array_type
+
+
 class NestedArray(Array):
     """
     A NestedArray is an array nested within a structured type (which are "void"
@@ -967,7 +840,11 @@ class UniTuple(IterableType, BaseTuple):
         self.count = count
         name = "(%s x %d)" % (dtype, count)
         super(UniTuple, self).__init__(name, param=True)
-        self.iterator_type = UniTupleIter(self)
+        self._iterator_type = UniTupleIter(self)
+
+    @property
+    def iterator_type(self):
+        return self._iterator_type
 
     def getitem(self, ind):
         return self.dtype, intp
@@ -992,24 +869,30 @@ class UniTuple(IterableType, BaseTuple):
     def types(self):
         return (self.dtype,) * self.count
 
-    def coerce(self, typingctx, other):
+    def unify(self, typingctx, other):
         """
         Unify UniTuples with their dtype
         """
         if isinstance(other, UniTuple) and len(self) == len(other):
             dtype = typingctx.unify_pairs(self.dtype, other.dtype)
-            return UniTuple(dtype=dtype, count=self.count)
+            if dtype != pyobject:
+                return UniTuple(dtype=dtype, count=self.count)
 
-        return NotImplemented
+    def can_convert_to(self, typingctx, other):
+        """
+        Convert this UniTuple to another one.
+        """
+        if isinstance(other, UniTuple) and len(self) == len(other):
+            return typingctx.can_convert(self.dtype, other.dtype)
 
 
-class UniTupleIter(IteratorType):
+class UniTupleIter(SimpleIteratorType):
 
     def __init__(self, unituple):
         self.unituple = unituple
-        self.yield_type = unituple.dtype
+        yield_type = unituple.dtype
         name = 'iter(%s)' % unituple
-        super(UniTupleIter, self).__init__(name, param=True)
+        super(UniTupleIter, self).__init__(name, yield_type)
 
     @property
     def key(self):
@@ -1041,21 +924,28 @@ class Tuple(BaseTuple):
     def __iter__(self):
         return iter(self.types)
 
-    def coerce(self, typingctx, other):
+    def unify(self, typingctx, other):
         """
         Unify elements of Tuples/UniTuples
         """
         # Other is UniTuple or Tuple
-        if isinstance(other, (UniTuple, Tuple)) and len(self) == len(other):
+        if isinstance(other, BaseTuple) and len(self) == len(other):
             unified = [typingctx.unify_pairs(ta, tb)
                        for ta, tb in zip(self, other)]
 
-            if any(t == pyobject for t in unified):
-                return NotImplemented
+            if all(t != pyobject for t in unified):
+                return Tuple(unified)
 
-            return Tuple(unified)
-
-        return NotImplemented
+    def can_convert_to(self, typingctx, other):
+        """
+        Convert this Tuple to another UniTuple or Tuple.
+        """
+        if isinstance(other, BaseTuple) and len(self) == len(other):
+            kinds = [typingctx.can_convert(ta, tb)
+                     for ta, tb in zip(self, other)]
+            if any(kind is None for kind in kinds):
+                return
+            return max(kinds)
 
 
 class CPointer(Type):
@@ -1115,39 +1005,48 @@ class Object(Type):
 class Optional(Type):
     def __init__(self, typ):
         assert typ != none
-        assert not isinstance(typ, Optional)
+        assert not isinstance(typ, (Optional, NoneType))
         self.type = typ
         name = "?%s" % typ
         super(Optional, self).__init__(name, param=True)
 
-    def post_init(self):
-        """
-        Install conversion from optional(T) to T
-        """
-        from numba.typeconv.rules import default_casting_rules as tcr
-        tcr.safe(self, self.type)
-        tcr.promote(self.type, self)
-        tcr.promote(none, self)
-
     @property
     def key(self):
         return self.type
 
-    def coerce(self, typingctx, other):
+    def can_convert_to(self, typingctx, other):
+        if isinstance(other, Optional):
+            return typingctx.can_convert(self.type, other.type)
+        else:
+            conv = typingctx.can_convert(self.type, other)
+            if conv is not None:
+                return max(conv, Conversion.safe)
+
+    def can_convert_from(self, typingctx, other):
+        if other is none:
+            return Conversion.promote
+        elif isinstance(other, Optional):
+            return typingctx.can_convert(other.type, self.type)
+        else:
+            conv = typingctx.can_convert(other, self.type)
+            if conv is not None:
+                return max(conv, Conversion.promote)
+
+    def unify(self, typingctx, other):
         if isinstance(other, Optional):
             unified = typingctx.unify_pairs(self.type, other.type)
-
         else:
             unified = typingctx.unify_pairs(self.type, other)
 
         if unified != pyobject:
-            return Optional(unified)
-
-        return NotImplemented
+            if isinstance(unified, Optional):
+                return unified
+            else:
+                return Optional(unified)
 
 
 class NoneType(Opaque):
-    def coerce(self, typingctx, other):
+    def unify(self, typingctx, other):
         """Turns anything to a Optional type
         """
         if isinstance(other, Optional):
@@ -1198,8 +1097,6 @@ class Slice3Type(Type):
     pass
 
 
-
-
 # Utils
 
 def is_int_tuple(x):
@@ -1213,16 +1110,15 @@ def is_int_tuple(x):
 
 # Short names
 
-
-pyobject = Opaque('pyobject')
+pyobject = PyObject('pyobject')
 ffi_forced_object = Opaque('ffi_forced_object')
 none = NoneType('none')
 Any = Phantom('any')
-string = Dummy('str')
+string = Opaque('str')
 
 # No operation is defined on voidptr
 # Can only pass it around
-voidptr = Opaque('void*')
+voidptr = RawPointer('void*')
 
 # For NRT GC
 meminfo_pointer = Opaque("MemInfo*")
