@@ -196,15 +196,16 @@ class BinOpFloorDiv(ConcreteTemplate):
 @builtin
 class BinOpPower(ConcreteTemplate):
     key = "**"
-    cases = [signature(types.intp, types.intp, types.intp)]
+    cases = [signature(op, op, op)
+             for op in (types.intp, types.int64)]
     cases += [signature(types.float64, types.float64, op)
-             for op in sorted(types.signed_domain)]
+              for op in sorted(types.signed_domain)]
     cases += [signature(types.float64, types.float64, op)
-             for op in sorted(types.unsigned_domain)]
+              for op in sorted(types.unsigned_domain)]
     cases += [signature(op, op, op)
-             for op in sorted(types.real_domain)]
+              for op in sorted(types.real_domain)]
     cases += [signature(op, op, op)
-             for op in sorted(types.complex_domain)]
+              for op in sorted(types.complex_domain)]
 
 
 class PowerBuiltin(BinOpPower):
@@ -409,6 +410,8 @@ def normalize_index(index):
         return types.intp if index.signed else types.uintp
 
 
+# XXX Should there be a base Sequence type for plain 1d sequences?
+
 @builtin
 class GetItemUniTuple(AbstractTemplate):
     key = "getitem"
@@ -419,6 +422,17 @@ class GetItemUniTuple(AbstractTemplate):
             return signature(tup.dtype, tup, normalize_index(idx))
 
 
+@builtin
+class GetItemFlat(AbstractTemplate):
+    key = "getitem"
+
+    def generic(self, args, kws):
+        obj, idx = args
+        if (isinstance(obj, types.NumpyFlatType)
+            and isinstance(idx, types.Integer)):
+            return signature(obj.array_type.dtype, obj, normalize_index(idx))
+
+
 @builtin
 class GetItemBuffer(AbstractTemplate):
     key = "getitem"
@@ -452,7 +466,9 @@ class GetItemBuffer(AbstractTemplate):
             if ary.ndim == 1:
                 res = ary.dtype
             elif not ary.slice_is_copy and ary.ndim > 1:
-                res = ary.copy(ndim=ary.ndim - 1)
+                # Left-index into a F-contiguous array gives a non-contiguous view
+                layout = 'C' if ary.layout == 'C' else 'A'
+                res = ary.copy(ndim=ary.ndim - 1, layout=layout)
             else:
                 return
 
@@ -487,7 +503,7 @@ class SetItemBuffer(AbstractTemplate):
         ary, idx, val = args
         if isinstance(ary, types.Buffer):
             if not ary.mutable:
-                raise TypeError("Immutable array")
+                raise TypeError("Cannot modify value of type %s" %(ary,))
             return signature(types.none, ary, normalize_index(idx), ary.dtype)
 
 
@@ -515,10 +531,23 @@ class Len(AbstractTemplate):
 
 #-------------------------------------------------------------------------------
 
+def normalize_shape(shape):
+    if isinstance(shape, types.UniTuple):
+        if isinstance(shape.dtype, types.Integer):
+            dimtype = types.intp if shape.dtype.signed else types.uintp
+            return types.UniTuple(dimtype, len(shape))
+
+    elif isinstance(shape, types.Tuple) and shape.count == 0:
+        return shape
+
+
 @builtin_attr
 class ArrayAttribute(AttributeTemplate):
     key = types.Array
 
+    def resolve_dtype(self, ary):
+        return types.DType(ary.dtype)
+
     def resolve_itemsize(self, ary):
         return types.intp
 
@@ -540,7 +569,58 @@ class ArrayAttribute(AttributeTemplate):
     def resolve_ctypes(self, ary):
         return types.ArrayCTypes(ary)
 
+    def resolve_flags(self, ary):
+        return types.ArrayFlags(ary)
+
+    def resolve_T(self, ary):
+        if ary.ndim <= 1:
+            retty = ary
+        else:
+            layout = {"C": "F", "F": "C"}.get(ary.layout, "A")
+            retty = ary.copy(layout=layout)
+        return retty
+
+    @bound_function("array.transpose")
+    def resolve_transpose(self, ary, args, kws):
+        assert not args
+        assert not kws
+        return signature(self.resolve_T(ary))
+
+    @bound_function("array.copy")
+    def resolve_copy(self, ary, args, kws):
+        assert not args
+        assert not kws
+        retty = ary.copy(layout="C")
+        return signature(retty)
+
+    @bound_function("array.reshape")
+    def resolve_reshape(self, ary, args, kws):
+        assert not kws
+        shape, = args
+        shape = normalize_shape(shape)
+        if shape is None:
+            return
+        if ary.layout == "C":
+            # Given order='C' (the only supported value), a C-contiguous
+            # array is always returned for a C-contiguous input.
+            layout = "C"
+        else:
+            layout = "A"
+        ndim = shape.count if isinstance(shape, types.BaseTuple) else 1
+        retty = ary.copy(ndim=ndim)
+        return signature(retty, shape)
+
+    @bound_function("array.view")
+    def resolve_view(self, ary, args, kws):
+        from .npydecl import _parse_dtype
+        assert not kws
+        dtype, = args
+        dtype = _parse_dtype(dtype)
+        retty = ary.copy(dtype=dtype)
+        return signature(retty, *args)
+
     def generic_resolve(self, ary, attr):
+        # Resolution of other attributes, for record arrays
         if isinstance(ary.dtype, types.Record):
             if attr in ary.dtype.fields:
                 return types.Array(ary.dtype.typeof(attr), ndim=ary.ndim,
@@ -555,6 +635,19 @@ class ArrayCTypesAttribute(AttributeTemplate):
         return types.uintp
 
 
+@builtin_attr
+class ArrayFlagsAttribute(AttributeTemplate):
+    key = types.ArrayFlags
+
+    def resolve_contiguous(self, ctflags):
+        return types.boolean
+
+    def resolve_c_contiguous(self, ctflags):
+        return types.boolean
+
+    def resolve_f_contiguous(self, ctflags):
+        return types.boolean
+
 
 @builtin_attr
 class NestedArrayAttribute(ArrayAttribute):
@@ -740,7 +833,8 @@ class Max(AbstractTemplate):
                 return
 
         retty = self.context.unify_types(*args)
-        return signature(retty, *args)
+        if retty is not None:
+            return signature(retty, *args)
 
 
 class Min(AbstractTemplate):
@@ -757,7 +851,8 @@ class Min(AbstractTemplate):
                 return
 
         retty = self.context.unify_types(*args)
-        return signature(retty, *args)
+        if retty is not None:
+            return signature(retty, *args)
 
 
 class Round(ConcreteTemplate):
