@@ -10,7 +10,7 @@ from ..numpy_support import (ufunc_find_matching_loop,
                              from_dtype)
 from ..numpy_support import version as numpy_version
 
-from ..typeinfer import TypingError
+from ..errors import TypingError
 
 registry = Registry()
 builtin = registry.register
@@ -598,6 +598,28 @@ class NdLinspace(AbstractTemplate):
 builtin_global(numpy.linspace, types.Function(NdLinspace))
 
 
+@builtin
+class NdFromBuffer(CallableTemplate):
+    key = numpy.frombuffer
+
+    def generic(self):
+        def typer(buffer, dtype=None):
+            if not isinstance(buffer, types.Buffer) or buffer.layout != 'C':
+                return
+            if dtype is None:
+                nb_dtype = types.float64
+            else:
+                nb_dtype = _parse_dtype(dtype)
+
+            if nb_dtype is not None:
+                return types.Array(dtype=nb_dtype, ndim=1, layout='C',
+                                   readonly=not buffer.mutable)
+
+        return typer
+
+builtin_global(numpy.frombuffer, types.Function(NdFromBuffer))
+
+
 # -----------------------------------------------------------------------------
 # Miscellaneous functions
 
@@ -623,12 +645,16 @@ class NdIndex(AbstractTemplate):
         assert not kws
 
         # Either ndindex(shape) or ndindex(*shape)
-        if len(args) == 1 and isinstance(args[0], types.UniTuple):
-            shape = list(args[0])
+        if len(args) == 1 and isinstance(args[0], types.BaseTuple):
+            tup = args[0]
+            if tup.count > 0 and not isinstance(tup, types.UniTuple):
+                # Heterogenous tuple
+                return
+            shape = list(tup)
         else:
             shape = args
 
-        if shape and all(isinstance(x, types.Integer) for x in shape):
+        if all(isinstance(x, types.Integer) for x in shape):
             iterator_type = types.NumpyNdIndexType(len(shape))
             return signature(iterator_type, *args)
 
