@@ -4,11 +4,12 @@ Define typing templates
 from __future__ import print_function, division, absolute_import
 
 import functools
-from .. import types, utils
-from ..typeinfer import TypingError
 from functools import reduce
 import operator
 
+from .. import types, utils
+from ..errors import TypingError, UntypedAttributeError
+
 
 class Signature(object):
     # XXX Perhaps the signature should be a BoundArguments, instead
@@ -120,113 +121,12 @@ def _sum_downcast(dists):
     return c
 
 
-class Rating(object):
-    __slots__ = 'promote', 'safe_convert', "unsafe_convert"
-
-    def __init__(self):
-        self.promote = 0
-        self.safe_convert = 0
-        self.unsafe_convert = 0
-
-    def astuple(self):
-        """Returns a tuple suitable for comparing with the worse situation
-        start first.
-        """
-        return (self.unsafe_convert, self.safe_convert, self.promote)
-
-    def __add__(self, other):
-        if type(self) is not type(other):
-            return NotImplemented
-        rsum = Rating()
-        rsum.promote = self.promote + other.promote
-        rsum.safe_convert = self.safe_convert + other.safe_convert
-        rsum.unsafe_convert = self.unsafe_convert + other.unsafe_convert
-        return rsum
-
-
-def _rate_arguments(context, actualargs, formalargs):
-    ratings = [Rating()]
-    for actual, formal in zip(actualargs, formalargs):
-        rate = Rating()
-        by = context.type_compatibility(actual, formal)
-        if by is None:
-            return None
-
-        if by == 'promote':
-            rate.promote += 1
-        elif by == 'safe':
-            rate.safe_convert += 1
-        elif by == 'unsafe':
-            rate.unsafe_convert += 1
-        elif by == 'exact':
-            pass
-        else:
-            raise Exception("unreachable", by)
-
-        ratings.append(rate)
-    return ratings
-
-
-def resolve_overload(context, key, cases, args, kws):
-    assert not kws, "Keyword arguments are not supported, yet"
-    # Rate each cases
-    candids = []
-    symm_ratings = []
-    for case in cases:
-        if len(args) == len(case.args):
-            ratings = _rate_arguments(context, args, case.args)
-            if ratings is not None:
-                combined = reduce(operator.add, ratings)
-                symm_ratings.append(combined.astuple())
-                candids.append(case)
-
-    # Find the best case
-    ordered = sorted(zip(symm_ratings, candids), key=lambda i: i[0])
-    if ordered:
-        if len(ordered) > 1:
-            (first, case1), (second, case2) = ordered[:2]
-            # Ambiguous overloading
-            # NOTE: we can have duplicate overloadings if e.g. some type
-            # aliases were used when declaring the supported signatures
-            # (typical example being "intp" and "int64" on a 64-bit build)
-            if first == second and case1 != case2:
-                ambiguous = []
-                for rate, case in ordered:
-                    if rate == first:
-                        ambiguous.append(case)
-
-                # Try to resolve promotion
-                # TODO: need to match this to the C overloading dispatcher
-                resolvable = resolve_ambiguous_resolution(context, ambiguous,
-                                                          args)
-                if resolvable:
-                    return resolvable
-
-                # Failed to resolve promotion
-                args = (key, args, '\n'.join(map(str, ambiguous)))
-                msg = "Ambiguous overloading for %s %s\n%s" % args
-                raise TypeError(msg)
-
-        return ordered[0][1]
-
-
-def resolve_ambiguous_resolution(context, cases, args):
-    """Uses asymmetric resolution to find the best version
-    """
-    ratings = []
-    for case in cases:
-        rates = _rate_arguments(context, args, case.args)
-        ratings.append((tuple(r.astuple() for r in rates), case))
-
-    return max(ratings, key=lambda x: x[0])[1]
-
-
 class FunctionTemplate(object):
     def __init__(self, context):
         self.context = context
 
     def _select(self, cases, args, kws):
-        selected = resolve_overload(self.context, self.key, cases, args, kws)
+        selected = self.context.resolve_overload(self.key, cases, args, kws)
         return selected
 
 
@@ -275,15 +175,16 @@ class CallableTemplate(FunctionTemplate):
         sig = typer(*args, **kws)
 
         # Unpack optional type if no matching signature
-        if sig is None and any(isinstance(x, types.Optional) for x in args):
-            def unpack_opt(x):
-                if isinstance(x, types.Optional):
-                    return x.type
-                else:
-                    return x
+        if sig is None:
+            if any(isinstance(x, types.Optional) for x in args):
+                def unpack_opt(x):
+                    if isinstance(x, types.Optional):
+                        return x.type
+                    else:
+                        return x
 
-            args = list(map(unpack_opt, args))
-            sig = typer(*args, **kws)
+                args = list(map(unpack_opt, args))
+                sig = typer(*args, **kws)
             if sig is None:
                 return
 
@@ -318,13 +219,6 @@ class ConcreteTemplate(FunctionTemplate):
         return self._select(cases, args, kws)
 
 
-class UntypedAttributeError(TypingError):
-    def __init__(self, value, attr):
-        msg = 'Unknown attribute "{attr}" of type {type}'.format(type=value,
-                                                              attr=attr)
-        super(UntypedAttributeError, self).__init__(msg)
-
-
 class AttributeTemplate(object):
     def __init__(self, context):
         self.context = context
